@@ -18,7 +18,7 @@ def create_executable():
     """Build executable using ``guidata.disthelpers``"""
     dist = Distribution()
     dist.setup(name="Example", version="1.1",
-               description=u"Embedding Spyder Qt shell",
+               description="Embedding Spyder Qt shell",
                script="example.pyw", target_name="example.exe")
     spyderlib.add_to_distribution(dist)
     #dist.add_modules('matplotlib')  # Uncomment if you need matplotlib
