@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2011 Pierre Raybaut
+# Copyright © 2011-2013 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
@@ -13,19 +13,24 @@ exclusively with non-GUI features configuration only
 sip API incompatibility issue in spyderlib's non-gui modules)
 """
 
+from __future__ import print_function
+
 import os.path as osp
 import os
 import sys
 
 # Local imports
 from spyderlib import __version__
+from spyderlib.utils import encoding
+from spyderlib.py3compat import (is_unicode, TEXT_TYPES, INT_TYPES, PY3,
+                                 to_text_string, is_text_string)
 
 
 #==============================================================================
 # Only for development
 #==============================================================================
 # To activate/deactivate certain things for development
-# SPYDER_DEV is (and *only* have to be) set in bootstrap.py
+# SPYDER_DEV is (and *only* has to be) set in bootstrap.py
 DEV = os.environ.get('SPYDER_DEV')
 
 # For testing purposes
@@ -45,21 +50,58 @@ def _get_debug_env():
     return int(debug_env)    
 DEBUG = _get_debug_env()
 
+def debug_print(message):
+    """Output debug messages to stdout"""
+    if DEBUG:
+        ss = STDOUT
+        print(message, file=ss)
 
 #==============================================================================
 # Configuration paths
 #==============================================================================
+# Spyder settings dir
 if TEST is None:
     SUBFOLDER = '.spyder%s' % __version__.split('.')[0]
 else:
     SUBFOLDER = 'spyder_test'
 
 
+# We can't have PY2 and PY3 settings in the same dir because:
+# 1. This leads to ugly crashes and freezes (e.g. by trying to
+#    embed a PY2 interpreter in PY3)
+# 2. We need to save the list of installed modules (for code
+#    completion) separately for each version
+if PY3:
+    SUBFOLDER = SUBFOLDER + '-py3'
+
+
+def get_home_dir():
+    """
+    Return user home directory
+    """
+    try:
+        # expanduser() returns a raw byte string which needs to be
+        # decoded with the codec that the OS is using to represent file paths.
+        path = encoding.to_unicode_from_fs(osp.expanduser('~'))
+    except:
+        path = ''
+    for env_var in ('HOME', 'USERPROFILE', 'TMP'):
+        if osp.isdir(path):
+            break
+        # os.environ.get() returns a raw byte string which needs to be
+        # decoded with the codec that the OS is using to represent environment
+        # variables.
+        path = encoding.to_unicode_from_fs(os.environ.get(env_var, ''))
+    if path:
+        return path
+    else:
+        raise RuntimeError('Please define environment variable $HOME')
+
+
 def get_conf_path(filename=None):
     """Return absolute path for configuration file with specified filename"""
     if TEST is None:
-        from spyderlib import userconfig
-        conf_dir = osp.join(userconfig.get_home_dir(), SUBFOLDER)
+        conf_dir = osp.join(get_home_dir(), SUBFOLDER)
     else:
          import tempfile
          conf_dir = osp.join(tempfile.gettempdir(), SUBFOLDER)
@@ -177,15 +219,19 @@ def get_translation(modname, dirname=None):
         _trans = gettext.translation(modname, locale_path, codeset="utf-8")
         lgettext = _trans.lgettext
         def translate_gettext(x):
-            if isinstance(x, unicode):
+            if not PY3 and is_unicode(x):
                 x = x.encode("utf-8")
-            return unicode(lgettext(x), "utf-8")
+            y = lgettext(x)
+            if is_text_string(y) and PY3:
+                return y
+            else:
+                return to_text_string(y, "utf-8")
         return translate_gettext
-    except IOError, _e:  # analysis:ignore
+    except IOError as _e:  # analysis:ignore
         #print "Not using translations (%s)" % _e
         def translate_dumb(x):
-            if not isinstance(x, unicode):
-                return unicode(x, "utf-8")
+            if not is_unicode(x):
+                return to_text_string(x, "utf-8")
             return x
         return translate_dumb
 
@@ -211,10 +257,11 @@ def get_supported_types():
     If you update this list, don't forget to update doc/variablexplorer.rst
     """
     from datetime import date
-    editable_types = [int, long, float, list, dict, tuple, str, unicode, date]
+    editable_types = [int, float, complex, list, dict, tuple, date
+                      ] + list(TEXT_TYPES) + list(INT_TYPES)
     try:
-        from numpy import ndarray, matrix
-        editable_types += [ndarray, matrix]
+        from numpy import ndarray, matrix, generic
+        editable_types += [ndarray, matrix, generic]
     except ImportError:
         pass
     picklable_types = editable_types[:]
@@ -232,4 +279,5 @@ CHECK_ALL = False #XXX: If True, this should take too much to compute...
 
 EXCLUDED_NAMES = ['nan', 'inf', 'infty', 'little_endian', 'colorbar_doc',
                   'typecodes', '__builtins__', '__main__', '__doc__', 'NaN',
-                  'Inf', 'Infinity', 'sctypes']
\ No newline at end of file
+                  'Inf', 'Infinity', 'sctypes', 'rcParams', 'rcParamsDefault',
+                  'sctypeNA', 'typeNA', 'False_', 'True_',]
