@@ -31,10 +31,23 @@ from spyderlib.qt.QtGui import (QWidget, QDialog, QListWidget, QListWidgetItem,
 from spyderlib.qt.QtCore import Qt, QSize, SIGNAL, SLOT, Slot
 from spyderlib.qt.compat import (to_qvariant, from_qvariant,
                                  getexistingdirectory, getopenfilename)
+from spyderlib.py3compat import to_text_string, is_text_string, getcwd
+
+
+class ConfigAccessMixin(object):
+    """Namespace for methods that access config storage"""
+    CONF_SECTION = None
+
+    def set_option(self, option, value):
+        CONF.set(self.CONF_SECTION, option, value)
+
+    def get_option(self, option, default=NoDefault):
+        return CONF.get(self.CONF_SECTION, option, default)
 
 
 class ConfigPage(QWidget):
-    """Configuration page base class"""
+    """Base class for configuration page in Preferences"""
+
     def __init__(self, parent, apply_callback=None):
         QWidget.__init__(self, parent)
         self.apply_callback = apply_callback
@@ -50,11 +63,11 @@ class ConfigPage(QWidget):
         self.load_from_conf()
         
     def get_name(self):
-        """Return page name"""
+        """Return configuration page name"""
         raise NotImplementedError
     
     def get_icon(self):
-        """Return page icon"""
+        """Return configuration page icon (24x24)"""
         raise NotImplementedError
     
     def setup_page(self):
@@ -203,8 +216,10 @@ class ConfigDialog(QDialog):
         self.emit(SIGNAL("size_change(QSize)"), self.size())
 
 
-class SpyderConfigPage(ConfigPage):
+class SpyderConfigPage(ConfigPage, ConfigAccessMixin):
     """Plugin configuration dialog box page widget"""
+    CONF_SECTION = None
+
     def __init__(self, parent):
         ConfigPage.__init__(self, parent,
                             apply_callback=lambda:
@@ -239,7 +254,7 @@ class SpyderConfigPage(ConfigPage):
         for lineedit in self.lineedits:
             if lineedit in self.validate_data and lineedit.isEnabled():
                 validator, invalid_msg = self.validate_data[lineedit]
-                text = unicode(lineedit.text())
+                text = to_text_string(lineedit.text())
                 if not validator(text):
                     QMessageBox.critical(self, self.get_name(),
                                      "%s:<br><b>%s</b>" % (invalid_msg, text),
@@ -249,35 +264,35 @@ class SpyderConfigPage(ConfigPage):
         
     def load_from_conf(self):
         """Load settings from configuration file"""
-        for checkbox, (option, default) in self.checkboxes.items():
+        for checkbox, (option, default) in list(self.checkboxes.items()):
             checkbox.setChecked(self.get_option(option, default))
             self.connect(checkbox, SIGNAL("clicked(bool)"),
                          lambda _foo, opt=option: self.has_been_modified(opt))
-        for radiobutton, (option, default) in self.radiobuttons.items():
+        for radiobutton, (option, default) in list(self.radiobuttons.items()):
             radiobutton.setChecked(self.get_option(option, default))
             self.connect(radiobutton, SIGNAL("toggled(bool)"),
                          lambda _foo, opt=option: self.has_been_modified(opt))
-        for lineedit, (option, default) in self.lineedits.items():
+        for lineedit, (option, default) in list(self.lineedits.items()):
             lineedit.setText(self.get_option(option, default))
             self.connect(lineedit, SIGNAL("textChanged(QString)"),
                          lambda _foo, opt=option: self.has_been_modified(opt))
-        for spinbox, (option, default) in self.spinboxes.items():
+        for spinbox, (option, default) in list(self.spinboxes.items()):
             spinbox.setValue(self.get_option(option, default))
             self.connect(spinbox, SIGNAL('valueChanged(int)'),
                          lambda _foo, opt=option: self.has_been_modified(opt))
-        for combobox, (option, default) in self.comboboxes.items():
+        for combobox, (option, default) in list(self.comboboxes.items()):
             value = self.get_option(option, default)
             for index in range(combobox.count()):
-                data = from_qvariant(combobox.itemData(index), unicode)
+                data = from_qvariant(combobox.itemData(index), to_text_string)
                 # For PyQt API v2, it is necessary to convert `data` to 
                 # unicode in case the original type was not a string, like an 
                 # integer for example (see spyderlib.qt.compat.from_qvariant):
-                if unicode(data) == unicode(value):
+                if to_text_string(data) == to_text_string(value):
                     break
             combobox.setCurrentIndex(index)
             self.connect(combobox, SIGNAL('currentIndexChanged(int)'),
                          lambda _foo, opt=option: self.has_been_modified(opt))
-        for (fontbox, sizebox), option in self.fontboxes.items():
+        for (fontbox, sizebox), option in list(self.fontboxes.items()):
             font = self.get_font(option)
             fontbox.setCurrentFont(font)
             sizebox.setValue(font.pointSize())
@@ -289,7 +304,7 @@ class SpyderConfigPage(ConfigPage):
                          lambda _foo, opt=property: self.has_been_modified(opt))
             self.connect(sizebox, SIGNAL('valueChanged(int)'),
                          lambda _foo, opt=property: self.has_been_modified(opt))
-        for clayout, (option, default) in self.coloredits.items():
+        for clayout, (option, default) in list(self.coloredits.items()):
             property = to_qvariant(option)
             edit = clayout.lineedit
             btn = clayout.colorbtn
@@ -299,7 +314,7 @@ class SpyderConfigPage(ConfigPage):
             self.connect(edit, SIGNAL("textChanged(QString)"),
                          lambda _foo, opt=option: self.has_been_modified(opt))
         for (clayout, cb_bold, cb_italic
-             ), (option, default) in self.scedits.items():
+             ), (option, default) in list(self.scedits.items()):
             edit = clayout.lineedit
             btn = clayout.colorbtn
             color, bold, italic = self.get_option(option, default)
@@ -317,25 +332,25 @@ class SpyderConfigPage(ConfigPage):
     
     def save_to_conf(self):
         """Save settings to configuration file"""
-        for checkbox, (option, _default) in self.checkboxes.items():
+        for checkbox, (option, _default) in list(self.checkboxes.items()):
             self.set_option(option, checkbox.isChecked())
-        for radiobutton, (option, _default) in self.radiobuttons.items():
+        for radiobutton, (option, _default) in list(self.radiobuttons.items()):
             self.set_option(option, radiobutton.isChecked())
-        for lineedit, (option, _default) in self.lineedits.items():
-            self.set_option(option, unicode(lineedit.text()))
-        for spinbox, (option, _default) in self.spinboxes.items():
+        for lineedit, (option, _default) in list(self.lineedits.items()):
+            self.set_option(option, to_text_string(lineedit.text()))
+        for spinbox, (option, _default) in list(self.spinboxes.items()):
             self.set_option(option, spinbox.value())
-        for combobox, (option, _default) in self.comboboxes.items():
+        for combobox, (option, _default) in list(self.comboboxes.items()):
             data = combobox.itemData(combobox.currentIndex())
-            self.set_option(option, from_qvariant(data, unicode))
-        for (fontbox, sizebox), option in self.fontboxes.items():
+            self.set_option(option, from_qvariant(data, to_text_string))
+        for (fontbox, sizebox), option in list(self.fontboxes.items()):
             font = fontbox.currentFont()
             font.setPointSize(sizebox.value())
             self.set_font(font, option)
-        for clayout, (option, _default) in self.coloredits.items():
-            self.set_option(option, unicode(clayout.lineedit.text()))
-        for (clayout, cb_bold, cb_italic), (option, _default) in self.scedits.items():
-            color = unicode(clayout.lineedit.text())
+        for clayout, (option, _default) in list(self.coloredits.items()):
+            self.set_option(option, to_text_string(clayout.lineedit.text()))
+        for (clayout, cb_bold, cb_italic), (option, _default) in list(self.scedits.items()):
+            color = to_text_string(clayout.lineedit.text())
             bold = cb_bold.isChecked()
             italic = cb_italic.isChecked()
             self.set_option(option, (color, bold, italic))
@@ -426,9 +441,9 @@ class SpyderConfigPage(ConfigPage):
 
     def select_directory(self, edit):
         """Select directory"""
-        basedir = unicode(edit.text())
+        basedir = to_text_string(edit.text())
         if not osp.isdir(basedir):
-            basedir = os.getcwdu()
+            basedir = getcwd()
         title = _("Select directory")
         directory = getexistingdirectory(self, title, basedir)
         if directory:
@@ -457,9 +472,9 @@ class SpyderConfigPage(ConfigPage):
 
     def select_file(self, edit, filters=None):
         """Select File"""
-        basedir = osp.dirname(unicode(edit.text()))
+        basedir = osp.dirname(to_text_string(edit.text()))
         if not osp.isdir(basedir):
-            basedir = os.getcwdu()
+            basedir = getcwd()
         if filters is None:
             filters = _("All files (*)")
         title = _("Select file")
@@ -605,36 +620,43 @@ class SpyderConfigPage(ConfigPage):
 
 
 class GeneralConfigPage(SpyderConfigPage):
+    """Config page that maintains reference to main Spyder window
+       and allows to specify page name and icon declaratively
+    """
     CONF_SECTION = None
+
+    NAME = None    # configuration page name, e.g. _("General")
+    ICON = None    # name of icon resource (24x24)
+
     def __init__(self, parent, main):
         SpyderConfigPage.__init__(self, parent)
         self.main = main
 
-    def set_option(self, option, value):
-        CONF.set(self.CONF_SECTION, option, value)
+    def get_name(self):
+        """Configuration page name"""
+        return self.NAME
+    
+    def get_icon(self):
+        """Loads page icon named by self.ICON"""
+        return get_icon(self.ICON)
 
-    def get_option(self, option, default=NoDefault):
-        return CONF.get(self.CONF_SECTION, option, default)
-            
     def apply_settings(self, options):
         raise NotImplementedError
 
 
 class MainConfigPage(GeneralConfigPage):
     CONF_SECTION = "main"
-    def get_name(self):
-        return _("General")
     
-    def get_icon(self):
-        return get_icon("genprefs.png")
+    NAME = _("General")
+    ICON = "genprefs.png"
     
     def setup_page(self):
         newcb = self.create_checkbox
 
         # --- Interface
         interface_group = QGroupBox(_("Interface"))
-        styles = [str(txt) for txt in QStyleFactory.keys()]
-        choices = zip(styles, [style.lower() for style in styles])
+        styles = [str(txt) for txt in list(QStyleFactory.keys())]
+        choices = list(zip(styles, [style.lower() for style in styles]))
         style_combo = self.create_combobox(_('Qt windows style'), choices,
                                            'windows_style',
                                            default=self.main.default_style)
@@ -665,15 +687,7 @@ class MainConfigPage(GeneralConfigPage):
         margins_layout.addWidget(margin_spin)
 
         # Decide if it's possible to activate or not singie instance mode
-        if os.name == 'nt':
-            pywin32_present = programs.is_module_installed('win32api')
-            if not pywin32_present:
-                self.set_option("single_instance", False)
-                single_instance_box.setEnabled(False)
-                tip = _("This feature requires the pywin32 module.\n"
-                        "It seems you don't have it installed.")
-                single_instance_box.setToolTip(tip)
-        elif sys.platform == "darwin" and 'Spyder.app' in __file__:
+        if sys.platform == "darwin" and 'Spyder.app' in __file__:
             self.set_option("single_instance", True)
             single_instance_box.setEnabled(False)
         
@@ -720,8 +734,7 @@ class MainConfigPage(GeneralConfigPage):
         debug_group = QGroupBox(_("Debugging"))
         popup_console_box = newcb(_("Pop up internal console when internal "
                                     "errors appear"),
-                                  'show_internal_console_if_traceback',
-                                  default=True)
+                                  'show_internal_console_if_traceback')
         
         debug_layout = QVBoxLayout()
         debug_layout.addWidget(popup_console_box)
@@ -740,11 +753,9 @@ class MainConfigPage(GeneralConfigPage):
 
 class ColorSchemeConfigPage(GeneralConfigPage):
     CONF_SECTION = "color_schemes"
-    def get_name(self):
-        return _("Syntax coloring")
     
-    def get_icon(self):
-        return get_icon("genprefs.png")
+    NAME = _("Syntax coloring")
+    ICON = "genprefs.png"
     
     def setup_page(self):
         tabs = QTabWidget()
@@ -754,6 +765,7 @@ class ColorSchemeConfigPage(GeneralConfigPage):
         fieldnames = {
                       "background":     _("Background:"),
                       "currentline":    _("Current line:"),
+                      "currentcell":    _("Current cell:"),
                       "occurence":      _("Occurence:"),
                       "ctrlclick":      _("Link:"),
                       "sideareas":      _("Side areas:"),
@@ -778,7 +790,7 @@ class ColorSchemeConfigPage(GeneralConfigPage):
                 option = "%s/%s" % (tabname, key)
                 value = self.get_option(option)
                 name = fieldnames[key]
-                if isinstance(value, basestring):
+                if is_text_string(value):
                     label, clayout = self.create_coloredit(name, option,
                                                            without_layout=True)
                     label.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
