@@ -25,10 +25,10 @@ import os
 import os.path as osp
 import re
 import sys
+import subprocess
 
 # Local imports
-from spyderlib.baseconfig import _, SCIENTIFIC_STARTUP
-from spyderlib.ipythonconfig import SUPPORTED_IPYTHON
+from spyderlib.baseconfig import SCIENTIFIC_STARTUP, _
 from spyderlib.config import CONF
 from spyderlib.utils import programs
 from spyderlib.utils.misc import (get_error_match, get_python_executable,
@@ -39,6 +39,7 @@ from spyderlib.widgets.externalshell.pythonshell import ExternalPythonShell
 from spyderlib.widgets.externalshell.systemshell import ExternalSystemShell
 from spyderlib.widgets.findreplace import FindReplace
 from spyderlib.plugins import SpyderPluginWidget, PluginConfigPage
+from spyderlib.py3compat import to_text_string, is_text_string, getcwd
 from spyderlib import dependencies
 
 MPL_REQVER = '>=1.0'
@@ -137,26 +138,15 @@ class ExternalConsoleConfigPage(PluginConfigPage):
                                'codecompletion/auto')
         case_comp_box = newcb(_("Case sensitive code completion"),
                               'codecompletion/case_sensitive')
-        show_single_box = newcb(_("Show single completion"),
-                               'codecompletion/show_single')
         comp_enter_box = newcb(_("Enter key selects completion"),
                                'codecompletion/enter_key')
-        calltips_box = newcb(_("Balloon tips"), 'calltips')
-        inspector_box = newcb(
-                  _("Automatic notification to object inspector"),
-                  'object_inspector', default=True,
-                  tip=_("If this option is enabled, object inspector\n"
-                        "will automatically show informations on functions\n"
-                        "entered in console (this is triggered when entering\n"
-                        "a left parenthesis after a valid function name)"))
+        calltips_box = newcb(_("Display balloon tips"), 'calltips')
         
         source_layout = QVBoxLayout()
         source_layout.addWidget(completion_box)
         source_layout.addWidget(case_comp_box)
-        source_layout.addWidget(show_single_box)
         source_layout.addWidget(comp_enter_box)
         source_layout.addWidget(calltips_box)
-        source_layout.addWidget(inspector_box)
         source_group.setLayout(source_layout)
 
         # UMD Group
@@ -204,11 +194,11 @@ class ExternalConsoleConfigPage(PluginConfigPage):
                                 "binary in which Spyder will run scripts:"))
         def_exec_radio = self.create_radiobutton(
                                 _("Default (i.e. the same as Spyder's)"),
-                                'pythonexecutable/default', True,
+                                'pythonexecutable/default', 
                                 button_group=pyexec_bg)
         self.cus_exec_radio = self.create_radiobutton(
                                 _("Use the following Python interpreter:"),
-                                'pythonexecutable/custom', False,
+                                'pythonexecutable/custom',
                                 button_group=pyexec_bg)
         if os.name == 'nt':
             filters = _("Executables")+" (*.exe)"
@@ -230,15 +220,6 @@ class ExternalConsoleConfigPage(PluginConfigPage):
         pyexec_layout.addWidget(pyexec_file)
         pyexec_group.setLayout(pyexec_layout)
         
-        # Startup Group
-        startup_group = QGroupBox(_("Startup"))
-        pystartup_box = newcb(_("Open a Python interpreter at startup"),
-                              'open_python_at_startup')
-        
-        startup_layout = QVBoxLayout()
-        startup_layout.addWidget(pystartup_box)
-        startup_group.setLayout(startup_layout)
-        
         # PYTHONSTARTUP replacement
         pystartup_group = QGroupBox(_("PYTHONSTARTUP replacement"))
         pystartup_bg = QButtonGroup(pystartup_group)
@@ -248,11 +229,11 @@ class ExternalConsoleConfigPage(PluginConfigPage):
                                    "the Python interpreter startup."))
         self.def_startup_radio = self.create_radiobutton(
                                         _("Default PYTHONSTARTUP script"),
-                                        'pythonstartup/default', True,
+                                        'pythonstartup/default',
                                         button_group=pystartup_bg)
         self.cus_startup_radio = self.create_radiobutton(
                                         _("Use the following startup script:"),
-                                        'pythonstartup/custom', False,
+                                        'pythonstartup/custom',
                                         button_group=pystartup_bg)
         pystartup_file = self.create_browsefile('', 'pythonstartup', '',
                                                 filters=_("Python scripts")+\
@@ -279,8 +260,8 @@ class ExternalConsoleConfigPage(PluginConfigPage):
                                  "to accelerate console startup."))
         monitor_label.setWordWrap(True)
         monitor_box = newcb(_("Enable monitor"), 'monitor/enabled')
-        for obj in (completion_box, case_comp_box, show_single_box,
-                    comp_enter_box, calltips_box):
+        for obj in (completion_box, case_comp_box, comp_enter_box,
+                    calltips_box):
             self.connect(monitor_box, SIGNAL("toggled(bool)"), obj.setEnabled)
             obj.setEnabled(self.get_option('monitor/enabled'))
         
@@ -299,7 +280,7 @@ class ExternalConsoleConfigPage(PluginConfigPage):
                          tip=_("This option will act on<br> "
                                "libraries such as Matplotlib, guidata "
                                "or ETS"))
-        if self.get_option('pythonexecutable/default', True):
+        if self.get_option('pythonexecutable/default'):
             interpreter = get_python_executable()
         else:
             interpreter = self.get_option('pythonexecutable')
@@ -406,7 +387,7 @@ class ExternalConsoleConfigPage(PluginConfigPage):
                              "user interfaces."))
         ets_label.setWordWrap(True)
         ets_edit = self.create_lineedit(_("ETS_TOOLKIT:"), 'ets_backend',
-                                        default='qt4', alignment=Qt.Horizontal)
+                                        alignment=Qt.Horizontal)
         
         ets_layout = QVBoxLayout()
         ets_layout.addWidget(ets_label)
@@ -422,8 +403,7 @@ class ExternalConsoleConfigPage(PluginConfigPage):
                     _("Display"))
         tabs.addTab(self.create_tab(monitor_group, source_group),
                     _("Introspection"))
-        tabs.addTab(self.create_tab(pyexec_group, startup_group,
-                                    pystartup_group, umd_group),
+        tabs.addTab(self.create_tab(pyexec_group, pystartup_group, umd_group),
                     _("Advanced settings"))
         tabs.addTab(self.create_tab(qt_group, mpl_group, ets_group),
                     _("External modules"))
@@ -451,22 +431,44 @@ class ExternalConsoleConfigPage(PluginConfigPage):
         """Custom Python executable value has been changed"""
         if not self.cus_exec_radio.isChecked():
             return
-        if not isinstance(pyexec, basestring):
-            pyexec = unicode(pyexec.toUtf8(), 'utf-8')
+        if not is_text_string(pyexec):
+            pyexec = to_text_string(pyexec.toUtf8(), 'utf-8')
         old_pyexec = self.get_option("pythonexecutable",
                                      get_python_executable())
         if pyexec != old_pyexec:
             self._auto_change_qt_api(pyexec)
+        self.warn_python_compatibility(pyexec)
 
     def python_executable_switched(self, custom):
         """Python executable default/custom radio button has been toggled"""
         def_pyexec = get_python_executable()
         cust_pyexec = self.pyexec_edit.text()
-        if not isinstance(cust_pyexec, basestring):
-            cust_pyexec = unicode(cust_pyexec.toUtf8(), 'utf-8')
+        if not is_text_string(cust_pyexec):
+            cust_pyexec = to_text_string(cust_pyexec.toUtf8(), 'utf-8')
         if def_pyexec != cust_pyexec:
             pyexec = cust_pyexec if custom else def_pyexec
             self._auto_change_qt_api(pyexec)
+            if custom:
+                self.warn_python_compatibility(cust_pyexec)
+
+    def warn_python_compatibility(self, pyexec):
+        spyder_version = sys.version_info[0]
+        try:
+            cmd = [pyexec, "-c", "import sys; print(sys.version_info[0])"]
+            # subprocess.check_output is not present in python2.6 and 3.0
+            process = subprocess.Popen(cmd, stdout=subprocess.PIPE)
+            console_version = int(process.communicate()[0])
+        except IOError:
+            console_version = spyder_version
+        if spyder_version != console_version:
+            QMessageBox.warning(self, _('Warning'),
+                _("You selected a <b>Python %d</b> interpreter for the console "
+                  "but Spyder is running on <b>Python %d</b>!.<br><br>"
+                  "Although this is possible, we recommend you to install and "
+                  "run Spyder directly with your selected interpreter, to avoid "
+                  "seeing false warnings and errors due to the incompatible "
+                  "syntax between these two Python versions."
+                  ) % (console_version, spyder_version), QMessageBox.Ok)
 
 
 class ExternalConsole(SpyderPluginWidget):
@@ -475,6 +477,7 @@ class ExternalConsole(SpyderPluginWidget):
     """
     CONF_SECTION = 'console'
     CONFIGWIDGET_CLASS = ExternalConsoleConfigPage
+    DISABLE_ACTIONS_WHEN_HIDDEN = False
     def __init__(self, parent, light_mode):
         SpyderPluginWidget.__init__(self, parent)
         self.light_mode = light_mode
@@ -496,15 +499,15 @@ class ExternalConsole(SpyderPluginWidget):
         # Python executable selection (initializing default values as well)
         executable = self.get_option('pythonexecutable',
                                      get_python_executable())
-        if self.get_option('pythonexecutable/default', True):
+        if self.get_option('pythonexecutable/default'):
             executable = get_python_executable()
-
+        
         # Python startup file selection
         if not osp.isfile(self.get_option('pythonstartup', '')):
             self.set_option('pythonstartup', SCIENTIFIC_STARTUP)
         # default/custom settings are mutually exclusive:
         self.set_option('pythonstartup/custom',
-                        not self.get_option('pythonstartup/default', False))
+                        not self.get_option('pythonstartup/default'))
         
         if not osp.isfile(executable):
             # This is absolutely necessary, in case the Python interpreter
@@ -640,7 +643,7 @@ class ExternalConsole(SpyderPluginWidget):
         if current_index == -1:
             return
         from spyderlib.widgets.externalshell import pythonshell
-        for index in [current_index]+range(self.tabwidget.count()):
+        for index in [current_index]+list(range(self.tabwidget.count())):
             shellwidget = self.tabwidget.widget(index)
             if isinstance(shellwidget, pythonshell.ExternalPythonShell):
                 if interpreter_only and not shellwidget.is_interpreter:
@@ -681,7 +684,7 @@ class ExternalConsole(SpyderPluginWidget):
         
     def run_script_in_current_shell(self, filename, wdir, args, debug):
         """Run script in current shell, if any"""
-        norm = lambda text: remove_backslashes(unicode(text))
+        norm = lambda text: remove_backslashes(to_text_string(text))
         line = "%s('%s'" % ('debugfile' if debug else 'runfile',
                             norm(filename))
         if args:
@@ -693,7 +696,8 @@ class ExternalConsole(SpyderPluginWidget):
             QMessageBox.warning(self, _('Warning'),
                 _("No Python shell is currently selected to run <b>%s</b>."
                   "<br><br>Please select or open a new Python interpreter "
-                  "and try again.") % osp.basename(filename), QMessageBox.Ok)
+                  "and try again."
+                  ) % osp.basename(norm(filename)), QMessageBox.Ok)
         else:
             self.visibility_changed(True)
             self.raise_()            
@@ -702,14 +706,14 @@ class ExternalConsole(SpyderPluginWidget):
         """Set current shell working directory"""
         shellwidget = self.__find_python_shell()
         if shellwidget is not None:
-            shellwidget.shell.set_cwd(unicode(directory))
+            shellwidget.shell.set_cwd(to_text_string(directory))
         
     def execute_python_code(self, lines, interpreter_only=False):
         """Execute Python code in an already opened Python interpreter"""
         shellwidget = self.__find_python_shell(
                                         interpreter_only=interpreter_only)
         if (shellwidget is not None) and (not shellwidget.is_ipykernel):
-            shellwidget.shell.execute_lines(unicode(lines))
+            shellwidget.shell.execute_lines(to_text_string(lines))
             self.activateWindow()
             shellwidget.shell.setFocus()
             return True
@@ -722,7 +726,7 @@ class ExternalConsole(SpyderPluginWidget):
         # prevent keyboard input from accidentally entering the editor
         # during repeated, rapid entry of debugging commands.    
         self.emit(SIGNAL("edit_goto(QString,int,QString,bool)"),
-                  fname, lineno, '',False)
+                  fname, lineno, '', False)
         if shellwidget.is_ipykernel:
             # Focus client widget, not kernel
             ipw = self.main.ipyconsole.get_focus_widget()
@@ -738,8 +742,8 @@ class ExternalConsole(SpyderPluginWidget):
             shellwidget.shell.set_spyder_breakpoints()    
     
     def start(self, fname, wdir=None, args='', interact=False, debug=False,
-              python=True, ipykernel=False, ipyclient=False,
-              python_args=''):
+              python=True, ipykernel=False, ipyclient=None,
+              give_ipyclient_focus=True, python_args=''):
         """
         Start new console
         
@@ -757,10 +761,10 @@ class ExternalConsole(SpyderPluginWidget):
                    (option "-u" is mandatory, see widgets.externalshell package)
         """
         # Note: fname is None <=> Python interpreter
-        if fname is not None and not isinstance(fname, basestring):
-            fname = unicode(fname)
-        if wdir is not None and not isinstance(wdir, basestring):
-            wdir = unicode(wdir)
+        if fname is not None and not is_text_string(fname):
+            fname = to_text_string(fname)
+        if wdir is not None and not is_text_string(wdir):
+            wdir = to_text_string(wdir)
         
         if fname is not None and fname in self.filenames:
             index = self.filenames.index(fname)
@@ -786,11 +790,11 @@ class ExternalConsole(SpyderPluginWidget):
         light_background = self.get_option('light_background')
         show_elapsed_time = self.get_option('show_elapsed_time')
         if python:
-            if self.get_option('pythonexecutable/default', True):
+            if self.get_option('pythonexecutable/default'):
                 pythonexecutable = get_python_executable()
             else:
                 pythonexecutable = self.get_option('pythonexecutable')
-            if self.get_option('pythonstartup/default', True) or ipykernel:
+            if self.get_option('pythonstartup/default') or ipykernel:
                 pythonstartup = None
             else:
                 pythonstartup = self.get_option('pythonstartup', None)
@@ -801,7 +805,7 @@ class ExternalConsole(SpyderPluginWidget):
                 mpl_backend = self.get_option('matplotlib/backend/value')
             else:
                 mpl_backend = None
-            ets_backend = self.get_option('ets_backend', 'qt4')
+            ets_backend = self.get_option('ets_backend')
             qt_api = self.get_option('qt/api')
             if qt_api not in ('pyqt', 'pyside'):
                 qt_api = None
@@ -890,14 +894,12 @@ class ExternalConsole(SpyderPluginWidget):
                             self.get_option('codecompletion/auto') )
         shellwidget.shell.set_codecompletion_case(
                             self.get_option('codecompletion/case_sensitive') )
-        shellwidget.shell.set_codecompletion_single(
-                            self.get_option('codecompletion/show_single') )
         shellwidget.shell.set_codecompletion_enter(
                             self.get_option('codecompletion/enter_key') )
         if python and self.inspector is not None:
             shellwidget.shell.set_inspector(self.inspector)
             shellwidget.shell.set_inspector_enabled(
-                                            self.get_option('object_inspector'))
+                               CONF.get('inspector', 'connect/python_console'))
         if self.historylog is not None:
             self.historylog.add_history(shellwidget.shell.history_filename)
             self.connect(shellwidget.shell,
@@ -913,8 +915,14 @@ class ExternalConsole(SpyderPluginWidget):
                              self.open_file_in_spyder)
             if fname is None:
                 if ipykernel:
+                    # Connect client to any possible error while starting the
+                    # kernel
+                    ipyclient.connect(shellwidget,
+                              SIGNAL("ipython_kernel_start_error(QString)"),
+                              lambda error: ipyclient.show_kernel_error(error))
+                    
                     # Detect if kernel and frontend match or not
-                    if self.get_option('pythonexecutable/custom', False):
+                    if self.get_option('pythonexecutable/custom'):
                         frontend_ver = programs.get_module_version('IPython')
                         if '0.13' in frontend_ver:
                             frontend_ver = '<1.0'
@@ -934,20 +942,24 @@ class ExternalConsole(SpyderPluginWidget):
                         tab_name = _("Kernel")
                         tab_icon1 = get_icon('ipython_console.png')
                         tab_icon2 = get_icon('ipython_console_t.png')
-                        if ipyclient:
-                            self.connect(shellwidget,
+                        self.connect(shellwidget,
                                      SIGNAL('create_ipython_client(QString)'),
-                                     lambda cf: self.create_ipyclient(
-                                             cf, kernel_widget=shellwidget))
+                                     lambda cf: self.register_ipyclient(cf,
+                                              ipyclient,
+                                              shellwidget,
+                                              give_focus=give_ipyclient_focus))
                     else:
-                        QMessageBox.critical(self,
-                                     _("Mismatch between kernel and frontend"),
-                                     _("Your IPython frontend and kernel "
-                                       "versions are <b>incompatible!!</b>"
-                                       "<br><br>"
-                                       "We're sorry but we can't create an "
-                                       "IPython console for you."
-                                      ), QMessageBox.Ok)
+                        shellwidget.emit(
+                          SIGNAL("ipython_kernel_start_error(QString)"),
+                          _("Either:"
+                            "<ol>"
+                            "<li>Your IPython frontend and kernel versions "
+                            "are <b>incompatible</b> or</li>"
+                            "<li>You <b>don't have</b> IPython installed in "
+                            "your external interpreter.</li>"
+                            "</ol>"
+                            "In any case, we're sorry but we can't create a "
+                            "console for you."))
                         shellwidget.deleteLater()
                         shellwidget = None
                         return
@@ -997,7 +1009,7 @@ class ExternalConsole(SpyderPluginWidget):
             self.activateWindow()
             shellwidget.shell.setFocus()
     
-    def set_ipykernel_attrs(self, connection_file, kernel_widget):
+    def set_ipykernel_attrs(self, connection_file, kernel_widget, kernel_id):
         """Add the pid of the kernel process to an IPython kernel tab"""
         # Set connection file
         kernel_widget.connection_file = connection_file
@@ -1009,27 +1021,46 @@ class ExternalConsole(SpyderPluginWidget):
         # connection file at the time Spyder exits.
         def cleanup_connection_file(connection_file):
             connection_file = osp.join(get_ipython_dir(), 'profile_default',
-                                   'security', connection_file)
+                                       'security', connection_file)
             try:
                 os.remove(connection_file)
             except OSError:
                 pass
         atexit.register(cleanup_connection_file, connection_file)            
         
-        # Set tab name
+        # Set tab name according to kernel_id
         index = self.get_shell_index_from_id(id(kernel_widget))
-        match = re.match('^kernel-(\d+).json', connection_file)
-        if match is not None:  # should not fail, but we never know...
-            text = unicode(self.tabwidget.tabText(index))
-            name = "%s %s" % (text, match.groups()[0])
-            self.tabwidget.setTabText(index, name)
+        text = to_text_string(self.tabwidget.tabText(index))
+        name = "%s %s" % (text, kernel_id)
+        self.tabwidget.setTabText(index, name)
     
-    def create_ipyclient(self, connection_file, kernel_widget):
-        """Create a new IPython client connected to a kernel just started"""
-        self.set_ipykernel_attrs(connection_file, kernel_widget)
-        self.main.ipyconsole.new_client(connection_file, id(kernel_widget))
-        # QApplication.restoreOverrideCursor()  # Stop busy cursor indication
-        # QApplication.processEvents()
+    def register_ipyclient(self, connection_file, ipyclient, kernel_widget,
+                           give_focus=True):
+        """
+        Register `ipyclient` to be connected to `kernel_widget`
+        """
+        # Check if our client already has a connection_file and kernel_widget_id
+        # which means that we are asking for a kernel restart
+        cf = ipyclient.connection_file
+        kwid = ipyclient.kernel_widget_id
+        if cf is not None and kwid is not None:
+            restart_kernel = True
+        else:
+            restart_kernel = False
+        
+        # Setting kernel widget attributes
+        match = re.match('^kernel-(\d+).json', connection_file)
+        kernel_id = match.groups()[0]
+        self.set_ipykernel_attrs(connection_file, kernel_widget, kernel_id)
+        
+        # Creating the client
+        client_name = kernel_id + '/A'
+        ipyconsole = self.main.ipyconsole
+        ipyclient.connection_file = connection_file
+        ipyclient.kernel_widget_id = id(kernel_widget)
+        ipyconsole.register_client(ipyclient, client_name,
+                                   restart=restart_kernel,
+                                   give_focus=give_focus)
         
     def open_file_in_spyder(self, fname, lineno):
         """Open file in Spyder's editor from remote process"""
@@ -1067,7 +1098,7 @@ class ExternalConsole(SpyderPluginWidget):
             index = self.tabwidget.currentIndex()
             fname = self.filenames[index]
             if fname:
-                title += ' - '+unicode(fname)
+                title += ' - ' + to_text_string(fname)
         return title
     
     def get_plugin_icon(self):
@@ -1084,29 +1115,29 @@ class ExternalConsole(SpyderPluginWidget):
     def get_plugin_actions(self):
         """Return a list of actions related to plugin"""
         interpreter_action = create_action(self,
-                            _("Open a Python &interpreter"), None,
+                            _("Open a &Python console"), None,
                             'python.png', triggered=self.open_interpreter)
         if os.name == 'nt':
             text = _("Open &command prompt")
             tip = _("Open a Windows command prompt")
         else:
-            text = _("Open &terminal")
-            tip = _("Open a terminal window inside Spyder")
-        terminal_action = create_action(self, text, None, 'cmdprompt.png', tip,
+            text = _("Open a &terminal")
+            tip = _("Open a terminal window")
+        terminal_action = create_action(self, text, None, None, tip,
                                         triggered=self.open_terminal)
         run_action = create_action(self,
                             _("&Run..."), None,
                             'run_small.png', _("Run a Python script"),
                             triggered=self.run_script)
 
-        interact_menu_actions = [interpreter_action]
+        consoles_menu_actions = [interpreter_action]
         tools_menu_actions = [terminal_action]
         self.menu_actions = [interpreter_action, terminal_action, run_action]
         
-        self.main.interact_menu_actions += interact_menu_actions
+        self.main.consoles_menu_actions += consoles_menu_actions
         self.main.tools_menu_actions += tools_menu_actions
         
-        return self.menu_actions+interact_menu_actions+tools_menu_actions
+        return self.menu_actions+consoles_menu_actions+tools_menu_actions
     
     def register_plugin(self):
         """Register plugin in Spyder's main window"""
@@ -1123,7 +1154,7 @@ class ExternalConsole(SpyderPluginWidget):
                          self.main.editor.load)
             self.connect(self, SIGNAL("edit_goto(QString,int,QString,bool)"),
                          lambda fname, lineno, word, processevents:
-                         self.main.editor.load(fname,lineno,word,
+                         self.main.editor.load(fname, lineno, word,
                                                processevents=processevents))
             self.connect(self.main.editor,
                          SIGNAL('run_in_current_extconsole(QString,QString,QString,bool)'),
@@ -1180,97 +1211,109 @@ class ExternalConsole(SpyderPluginWidget):
     
     def apply_plugin_settings(self, options):
         """Apply configuration file's plugin settings"""
-        font = self.get_plugin_font()
-        showtime = self.get_option('show_elapsed_time')
-        icontext = self.get_option('show_icontext')
-        calltips = self.get_option('calltips')
-        inspector = self.get_option('object_inspector')
-        wrap = self.get_option('wrap')
-        compauto = self.get_option('codecompletion/auto')
-        case_comp = self.get_option('codecompletion/case_sensitive')
-        show_single = self.get_option('codecompletion/show_single')
-        compenter = self.get_option('codecompletion/enter_key')
-        mlc = self.get_option('max_line_count')
+        font_n = 'plugin_font'
+        font_o = self.get_plugin_font()
+        showtime_n = 'show_elapsed_time'
+        showtime_o = self.get_option(showtime_n)
+        icontext_n = 'show_icontext'
+        icontext_o = self.get_option(icontext_n)
+        calltips_n = 'calltips'
+        calltips_o = self.get_option(calltips_n)
+        inspector_n = 'connect_to_oi'
+        inspector_o = CONF.get('inspector', 'connect/python_console')
+        wrap_n = 'wrap'
+        wrap_o = self.get_option(wrap_n)
+        compauto_n = 'codecompletion/auto'
+        compauto_o = self.get_option(compauto_n)
+        case_comp_n = 'codecompletion/case_sensitive'
+        case_comp_o = self.get_option(case_comp_n)
+        compenter_n = 'codecompletion/enter_key'
+        compenter_o = self.get_option(compenter_n)
+        mlc_n = 'max_line_count'
+        mlc_o = self.get_option(mlc_n)
         for shellwidget in self.shellwidgets:
-            shellwidget.shell.set_font(font)
-            shellwidget.set_elapsed_time_visible(showtime)
-            shellwidget.set_icontext_visible(icontext)
-            shellwidget.shell.set_calltips(calltips)
-            if isinstance(shellwidget.shell, ExternalPythonShell):
-                shellwidget.shell.set_inspector_enabled(inspector)
-            shellwidget.shell.toggle_wrap_mode(wrap)
-            shellwidget.shell.set_codecompletion_auto(compauto)
-            shellwidget.shell.set_codecompletion_case(case_comp)
-            shellwidget.shell.set_codecompletion_single(show_single)
-            shellwidget.shell.set_codecompletion_enter(compenter)
-            shellwidget.shell.setMaximumBlockCount(mlc)
+            if font_n in options:
+                shellwidget.shell.set_font(font_o)
+                completion_size = CONF.get('shell_appearance',
+                                           'completion/size')
+                comp_widget = shellwidget.shell.completion_widget
+                comp_widget.setup_appearance(completion_size, font_o)
+            if showtime_n in options:
+                shellwidget.set_elapsed_time_visible(showtime_o)
+            if icontext_n in options:
+                shellwidget.set_icontext_visible(icontext_o)
+            if calltips_n in options:
+                shellwidget.shell.set_calltips(calltips_o)
+            if inspector_n in options:
+                if isinstance(shellwidget, ExternalPythonShell):
+                    shellwidget.shell.set_inspector_enabled(inspector_o)
+            if wrap_n in options:
+                shellwidget.shell.toggle_wrap_mode(wrap_o)
+            if compauto_n in options:
+                shellwidget.shell.set_codecompletion_auto(compauto_o)
+            if case_comp_n in options:
+                shellwidget.shell.set_codecompletion_case(case_comp_o)
+            if compenter_n in options:
+                shellwidget.shell.set_codecompletion_enter(compenter_o)
+            if mlc_n in options:
+                shellwidget.shell.setMaximumBlockCount(mlc_o)
+    
+    #------ SpyderPluginMixin API ---------------------------------------------
+    def toggle_view(self, checked):
+        """Toggle view"""
+        if checked:
+            self.dockwidget.show()
+            self.dockwidget.raise_()
+            # Start a console in case there are none shown
+            from spyderlib.widgets.externalshell import pythonshell
+            consoles = None
+            for sw in self.shellwidgets:
+                if isinstance(sw, pythonshell.ExternalPythonShell):
+                    if not sw.is_ipykernel:
+                        consoles = True
+                        break
+            if not consoles:
+                self.open_interpreter()
+        else:
+            self.dockwidget.hide()
     
     #------ Public API ---------------------------------------------------------
-    def open_interpreter_at_startup(self):
-        """Open an interpreter or an IPython kernel at startup"""
-        if self.get_option('open_python_at_startup', True):
-            self.open_interpreter()
-        if CONF.get('ipython_console', 'open_ipython_at_startup', False):
-            if self.main.ipyconsole is None:
-                suffix = SUPPORTED_IPYTHON.replace(';', _(' and ')
-                                         ).replace('<', '&lt;'
-                                         ).replace('>', '&gt;')
-                if programs.is_module_installed('IPython'):
-                    import IPython
-                    suffix += _('<br><u>Installed version</u>: %s'
-                                ) % IPython.__version__
-                QMessageBox.critical(self, _('IPython console'),
-                    _("Unable to open IPython console because no supported "
-                      "IPython version was found.<br><br>"
-                      "<u>Supported IPython versions</u>: %s") % suffix,
-                    QMessageBox.Ok)
-                if not self.get_option('open_python_at_startup'):
-                    self.open_interpreter()
-            else:
-                self.start_ipykernel()
-
     def open_interpreter(self, wdir=None):
         """Open interpreter"""
         if wdir is None:
-            wdir = os.getcwdu()
+            wdir = getcwd()
         if not self.main.light:
             self.visibility_changed(True)
-        self.start(fname=None, wdir=unicode(wdir), args='',
+        self.start(fname=None, wdir=to_text_string(wdir), args='',
                    interact=True, debug=False, python=True)
-        
-    def start_ipykernel(self, wdir=None, create_client=True):
+    
+    def start_ipykernel(self, client, wdir=None, give_focus=True):
         """Start new IPython kernel"""
-        if create_client and not self.get_option('monitor/enabled'):
+        if not self.get_option('monitor/enabled'):
             QMessageBox.warning(self, _('Open an IPython console'),
                 _("The console monitor was disabled: the IPython kernel will "
                   "be started as expected, but an IPython console will have "
                   "to be connected manually to the kernel."), QMessageBox.Ok)
         
-        # Add a WaitCursor visual indication, because it takes too much time
-        # to display a new console (3 to 5 secs). It's stopped in
-        # create_ipyclient
-        # QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-        # QApplication.processEvents()
-        
         if wdir is None:
-            wdir = os.getcwdu()
+            wdir = getcwd()
         self.main.ipyconsole.visibility_changed(True)
-        self.start(fname=None, wdir=unicode(wdir), args='',
-                   interact=True, debug=False, python=True,
-                   ipykernel=True, ipyclient=create_client)
+        self.start(fname=None, wdir=to_text_string(wdir), args='',
+                   interact=True, debug=False, python=True, ipykernel=True,
+                   ipyclient=client, give_ipyclient_focus=give_focus)
 
     def open_terminal(self, wdir=None):
         """Open terminal"""
         if wdir is None:
-            wdir = os.getcwdu()
-        self.start(fname=None, wdir=unicode(wdir), args='',
+            wdir = getcwd()
+        self.start(fname=None, wdir=to_text_string(wdir), args='',
                    interact=True, debug=False, python=False)
         
     def run_script(self):
         """Run a Python script"""
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
         filename, _selfilter = getopenfilename(self, _("Run Python script"),
-                os.getcwdu(), _("Python scripts")+" (*.py ; *.pyw ; *.ipy)")
+                getcwd(), _("Python scripts")+" (*.py ; *.pyw ; *.ipy)")
         self.emit(SIGNAL('redirect_stdio(bool)'), True)
         if filename:
             self.start(fname=filename, wdir=None, args='',
@@ -1284,7 +1327,7 @@ class ExternalConsole(SpyderPluginWidget):
                                   QLineEdit.Normal,
                                   ", ".join(self.get_option('umd/namelist')))
         if valid:
-            arguments = unicode(arguments)
+            arguments = to_text_string(arguments)
             if arguments:
                 namelist = arguments.replace(' ', '').split(',')
                 fixed_namelist = [module_name for module_name in namelist
@@ -1305,7 +1348,7 @@ class ExternalConsole(SpyderPluginWidget):
         
     def go_to_error(self, text):
         """Go to error if relevant"""
-        match = get_error_match(unicode(text))
+        match = get_error_match(to_text_string(text))
         if match:
             fname, lnb = match.groups()
             self.emit(SIGNAL("edit_goto(QString,int,QString)"),
@@ -1320,7 +1363,8 @@ class ExternalConsole(SpyderPluginWidget):
             if mimedata2url(source):
                 pathlist = mimedata2url(source)
                 shellwidget = self.tabwidget.currentWidget()
-                if all([is_python_script(unicode(qstr)) for qstr in pathlist]):
+                if all([is_python_script(to_text_string(qstr))
+                        for qstr in pathlist]):
                     event.acceptProposedAction()
                 elif shellwidget is None or not shellwidget.is_running():
                     event.ignore()
@@ -1338,16 +1382,16 @@ class ExternalConsole(SpyderPluginWidget):
         shellwidget = self.tabwidget.currentWidget()
         if source.hasText():
             qstr = source.text()
-            if is_python_script(unicode(qstr)):
+            if is_python_script(to_text_string(qstr)):
                 self.start(qstr)
             elif shellwidget:
                 shellwidget.shell.insert_text(qstr)
         elif source.hasUrls():
             pathlist = mimedata2url(source)
-            if all([is_python_script(unicode(qstr)) for qstr in pathlist]):
+            if all([is_python_script(to_text_string(qstr)) 
+                    for qstr in pathlist]):
                 for fname in pathlist:
                     self.start(fname)
             elif shellwidget:
                 shellwidget.shell.drop_pathlist(pathlist)
         event.acceptProposedAction()
-
