@@ -10,7 +10,7 @@ from spyderlib.qt.QtGui import (QHBoxLayout, QVBoxLayout, QLabel, QSizePolicy,
                                 QMenu, QToolButton, QGroupBox, QFontComboBox,
                                 QActionGroup, QFontDialog, QWidget, QComboBox,
                                 QLineEdit, QMessageBox)
-from spyderlib.qt.QtCore import SIGNAL, QUrl, QTimer, QThread
+from spyderlib.qt.QtCore import SIGNAL, QUrl, QThread
 
 import re
 import os.path as osp
@@ -20,7 +20,7 @@ import sys
 # Local imports
 from spyderlib import dependencies
 from spyderlib.baseconfig import get_conf_path, _
-from spyderlib.ipythonconfig import IPYTHON_QT_MODULE, SUPPORTED_IPYTHON
+from spyderlib.ipythonconfig import IPYTHON_QT_INSTALLED
 from spyderlib.config import CONF
 from spyderlib.guiconfig import get_color_scheme, get_font, set_font
 from spyderlib.utils import programs
@@ -32,10 +32,11 @@ from spyderlib.widgets.findreplace import FindReplace
 from spyderlib.widgets.browser import WebView
 from spyderlib.widgets.externalshell.pythonshell import ExtPythonShellWidget
 from spyderlib.plugins import SpyderPluginWidget, PluginConfigPage
+from spyderlib.py3compat import to_text_string, get_meth_class_inst
 
 #XXX: Hardcoded dependency on optional IPython plugin component
 #     that requires the hack to make this work without IPython
-if programs.is_module_installed(IPYTHON_QT_MODULE, SUPPORTED_IPYTHON):
+if IPYTHON_QT_INSTALLED:
     from spyderlib.widgets.ipython import IPythonControlWidget
 else:
     IPythonControlWidget = None  # analysis:ignore
@@ -44,7 +45,8 @@ else:
 if programs.is_module_installed('sphinx', '>=0.6.6'):
     sphinx_version = programs.get_module_version('sphinx')
     from spyderlib.utils.inspector.sphinxify import (CSS_PATH, sphinxify,
-                                                     warning, generate_context)
+                                                     warning, generate_context,
+                                                     usage)
 else:
     sphinxify = sphinx_version = None  # analysis:ignore
 
@@ -66,11 +68,13 @@ class ObjectComboBox(EditableComboBox):
         
     def is_valid(self, qstr=None):
         """Return True if string is valid"""
+        if not self.object_inspector.source_is_console():
+            return True
         if qstr is None:
             qstr = self.currentText()
         if not re.search('^[a-zA-Z0-9_\.]*$', str(qstr), 0):
             return False
-        objtxt = unicode(qstr)
+        objtxt = to_text_string(qstr)
         if self.object_inspector.get_option('automatic_import'):
             shell = self.object_inspector.internal_shell
             if shell is not None:
@@ -89,27 +93,67 @@ class ObjectComboBox(EditableComboBox):
         
     def validate_current_text(self):
         self.validate(self.currentText())
+    
+    def validate(self, qstr, editing=True):
+        """Reimplemented to avoid formatting actions"""
+        valid = self.is_valid(qstr)
+        if self.hasFocus() and valid is not None:
+            if editing:
+                # Combo box text is being modified: invalidate the entry
+                self.show_tip(self.tips[valid])
+                self.emit(SIGNAL('valid(bool)'), False)
+            else:
+                # A new item has just been selected
+                if valid:
+                    self.selected()
+                else:
+                    self.emit(SIGNAL('valid(bool)'), False)
+        else:
+            self.set_default_style()
+
 
 class ObjectInspectorConfigPage(PluginConfigPage):
     def setup_page(self):
-        sourcecode_group = QGroupBox(_("Source code"))
-        wrap_mode_box = self.create_checkbox(_("Wrap lines"), 'wrap')
-        names = CONF.get('color_schemes', 'names')
-        choices = zip(names, names)
-        cs_combo = self.create_combobox(_("Syntax color scheme: "),
-                                        choices, 'color_scheme_name')
-
-        sourcecode_layout = QVBoxLayout()
-        sourcecode_layout.addWidget(wrap_mode_box)
-        sourcecode_layout.addWidget(cs_combo)
-        sourcecode_group.setLayout(sourcecode_layout)
-        
+        # Fonts group        
         plain_text_font_group = self.create_fontgroup(option=None,
                                     text=_("Plain text font style"),
                                     fontfilters=QFontComboBox.MonospacedFonts)
         rich_text_font_group = self.create_fontgroup(option='rich_text',
                                 text=_("Rich text font style"))
-                                
+        
+        # Connections group
+        connections_group = QGroupBox(_("Automatic connections"))
+        connections_label = QLabel(_("The Object Inspector can automatically "
+                                     "show an object's help information after "
+                                     "a left parenthesis is written next to it. "
+                                     "Below you can decide to which plugin "
+                                     "you want to connect it to turn on this "
+                                     "feature."))
+        connections_label.setWordWrap(True)
+        editor_box = self.create_checkbox(_("Editor"), 'connect/editor')
+        rope_installed = programs.is_module_installed('rope')
+        jedi_installed = programs.is_module_installed('jedi', '>=0.8.0')
+        editor_box.setEnabled(rope_installed or jedi_installed)
+        # TODO: Don't forget to add Jedi here
+        if not rope_installed:
+            rope_tip = _("This feature requires the Rope library.\n"
+                         "It seems you don't have it installed.")
+            editor_box.setToolTip(rope_tip)
+        python_box = self.create_checkbox(_("Python Console"),
+                                          'connect/python_console')
+        ipython_box = self.create_checkbox(_("IPython Console"),
+                                           'connect/ipython_console')
+        ipython_installed = programs.is_module_installed('IPython', '>=0.13')
+        ipython_box.setEnabled(ipython_installed)
+        
+        connections_layout = QVBoxLayout()
+        connections_layout.addWidget(connections_label)
+        connections_layout.addWidget(editor_box)
+        connections_layout.addWidget(python_box)
+        connections_layout.addWidget(ipython_box)
+        connections_group.setLayout(connections_layout)
+        
+        # Features group
         features_group = QGroupBox(_("Additional features"))
         math_box = self.create_checkbox(_("Render mathematical equations"),
                                         'math')
@@ -127,9 +171,24 @@ class ObjectInspectorConfigPage(PluginConfigPage):
         features_layout.addWidget(math_box)
         features_group.setLayout(features_layout)
         
+        # Source code group
+        sourcecode_group = QGroupBox(_("Source code"))
+        wrap_mode_box = self.create_checkbox(_("Wrap lines"), 'wrap')
+        names = CONF.get('color_schemes', 'names')
+        choices = list(zip(names, names))
+        cs_combo = self.create_combobox(_("Syntax color scheme: "),
+                                        choices, 'color_scheme_name')
+
+        sourcecode_layout = QVBoxLayout()
+        sourcecode_layout.addWidget(wrap_mode_box)
+        sourcecode_layout.addWidget(cs_combo)
+        sourcecode_group.setLayout(sourcecode_layout)
+        
+        # Final layout
         vlayout = QVBoxLayout()
         vlayout.addWidget(rich_text_font_group)
         vlayout.addWidget(plain_text_font_group)
+        vlayout.addWidget(connections_group)
         vlayout.addWidget(features_group)
         vlayout.addWidget(sourcecode_group)
         vlayout.addStretch(1)
@@ -177,7 +236,7 @@ class PlainText(QWidget):
         # Read-only editor
         self.editor = codeeditor.CodeEditor(self)
         self.editor.setup_editor(linenumbers=False, language='py',
-                                 scrollflagarea=False)
+                                 scrollflagarea=False, edge_line=False)
         self.connect(self.editor, SIGNAL("focus_changed()"),
                      lambda: self.emit(SIGNAL("focus_changed()")))
         self.editor.setReadOnly(True)
@@ -221,32 +280,44 @@ class SphinxThread(QThread):
     
     Parameters
     ----------
-    text : dict
+    doc : dict
         A dict containing the doc string components to be rendered.
+        See spyderlib.utils.dochelpers.getdoc for description.
     html_text_no_doc : unicode
         Text to be rendered if doc string cannot be extracted.
     math_option : bool
         Use LaTeX math rendering.
         
     """
-    def __init__(self, text={}, html_text_no_doc='', math_option=False):
+    def __init__(self, html_text_no_doc=''):
         super(SphinxThread, self).__init__()
-        self.text = text
         self.html_text_no_doc = html_text_no_doc
-        self.math_option = math_option
+        self.doc = {}
+        self.math_option = False
+        
+    def render(self, doc, math_option=False):
+        """Start thread to render given doc string dictionary"""
+        # If the thread is already running wait for it to finish before
+        # starting it again.
+        if self.wait():
+            self.doc = doc
+            self.math_option = math_option
+            # This causes run() to be executed in separate thread
+            self.start()
 
     def run(self):
         html_text = self.html_text_no_doc
-        text = self.text
-        if text is not None and text['docstring'] != '':
+        doc = self.doc
+        if doc is not None and doc['docstring'] != '':
             try:
-                context = generate_context(name=text['name'],
-                                           argspec=text['argspec'],
-                                           note=text['note'],
+                context = generate_context(name=doc['name'],
+                                           argspec=doc['argspec'],
+                                           note=doc['note'],
                                            math=self.math_option)
-                html_text = sphinxify(text['docstring'], context)
-            except Exception, error:
-                self.emit(SIGNAL('error_msg(QString)'), unicode(error))
+                html_text = sphinxify(doc['docstring'], context)
+            except Exception as error:
+                self.emit(SIGNAL('error_msg(QString)'),
+                          to_text_string(error))
                 return
         self.emit(SIGNAL('html_ready(QString)'), html_text)
 
@@ -257,7 +328,7 @@ class ObjectInspector(SpyderPluginWidget):
     """
     CONF_SECTION = 'inspector'
     CONFIGWIDGET_CLASS = ObjectInspectorConfigPage
-    LOG_PATH = get_conf_path('.inspector')
+    LOG_PATH = get_conf_path(CONF_SECTION)
     def __init__(self, parent):
         SpyderPluginWidget.__init__(self, parent)
         
@@ -300,7 +371,7 @@ class ObjectInspector(SpyderPluginWidget):
         # locked = disable link with Console
         self.locked = False
         self._last_texts = [None, None]
-        self._last_rope_data = None
+        self._last_editor_doc = None
         
         # Object name
         layout_edit = QHBoxLayout()
@@ -315,7 +386,8 @@ class ObjectInspector(SpyderPluginWidget):
         self.source_combo.addItems([_("Console"), _("Editor")])
         self.connect(self.source_combo, SIGNAL('currentIndexChanged(int)'),
                      self.source_changed)
-        if not programs.is_module_installed('rope'):
+        if (not programs.is_module_installed('rope') and 
+                not programs.is_module_installed('jedi', '>=0.7.0')):
             self.source_combo.hide()
             source_label.hide()
         layout_edit.addWidget(self.source_combo)
@@ -396,9 +468,8 @@ class ObjectInspector(SpyderPluginWidget):
         if sphinxify is None:
             self._sphinx_thread = None
         else:
-            self._sphinx_thread = SphinxThread(text={},
-                                  html_text_no_doc=warning(self.no_doc_string),
-                                  math_option=self.get_option('math'))
+            self._sphinx_thread = SphinxThread(
+                                  html_text_no_doc=warning(self.no_doc_string))
             self.connect(self._sphinx_thread, SIGNAL('html_ready(QString)'), 
                          self._on_sphinx_thread_html_ready)
             self.connect(self._sphinx_thread, SIGNAL('error_msg(QString)'),
@@ -443,8 +514,9 @@ class ObjectInspector(SpyderPluginWidget):
         """Refresh widget"""
         if self._starting_up:
             self._starting_up = False
-            QTimer.singleShot(5000, self.refresh_plugin)
-        self.set_object_text(None, force_refresh=False)
+            if sphinxify is not None:
+                self.switch_to_rich_text()
+            self.show_intro_message()
 
     def apply_plugin_settings(self, options):
         """Apply configuration file's plugin settings"""
@@ -452,6 +524,7 @@ class ObjectInspector(SpyderPluginWidget):
         color_scheme_o = get_color_scheme(self.get_option(color_scheme_n))
         font_n = 'plugin_font'
         font_o = self.get_plugin_font()
+        connect_n = 'connect_to_oi'
         rich_font_n = 'rich_text'
         rich_font_o = self.get_plugin_font('rich_text')
         wrap_n = 'wrap'
@@ -459,6 +532,7 @@ class ObjectInspector(SpyderPluginWidget):
         self.wrap_action.setChecked(wrap_o)
         math_n = 'math'
         math_o = self.get_option(math_n)
+        
         if font_n in options:
             scs = color_scheme_o if color_scheme_n in options else None
             self.set_plain_text_font(font_o, color_scheme=scs)
@@ -470,6 +544,12 @@ class ObjectInspector(SpyderPluginWidget):
             self.toggle_wrap_mode(wrap_o)
         if math_n in options:
             self.toggle_math_mode(math_o)
+
+        # To make auto-connection changes take place instantly
+        self.main.editor.apply_plugin_settings(options=[connect_n])
+        self.main.extconsole.apply_plugin_settings(options=[connect_n])
+        if self.main.ipyconsole is not None:
+            self.main.ipyconsole.apply_plugin_settings(options=[connect_n])
         
     #------ Public API (related to inspector's source) -------------------------
     def source_is_console(self):
@@ -517,10 +597,32 @@ class ObjectInspector(SpyderPluginWidget):
             func = cb[0]
             args = cb[1:]
             func(*args)
-            if func.im_self is self.rich_text:
+            if get_meth_class_inst(func) is self.rich_text:
                 self.switch_to_rich_text()
             else:
                 self.switch_to_plain_text()
+    
+    def show_intro_message(self):
+        intro_message = _("Here you can get help of any object by pressing "
+                          "%s in front of it, either on the Editor or the "
+                          "Console.%s"
+                          "Help can also be shown automatically after writing "
+                          "a left parenthesis next to an object. You can "
+                          "activate this behavior in %s.")
+        prefs = _("Preferences > Object Inspector")
+        if self.is_rich_text_mode():
+            title = _("Usage")
+            intro_message = intro_message % ("<b>Ctrl+I</b>", "<br><br>",
+                                             "<i>"+prefs+"</i>")
+            self.set_rich_text_html(usage(title, intro_message),
+                                    QUrl.fromLocalFile(CSS_PATH))
+        else:
+            install_sphinx = "\n\n%s" % _("Please consider installing Sphinx "
+                                          "to get documentation rendered in "
+                                          "rich text.")
+            intro_message = intro_message % ("Ctrl+I", "\n\n", prefs)
+            intro_message += install_sphinx
+            self.set_plain_text(intro_message, is_code=False)
         
     #------ Public API (related to rich/plain text widgets) --------------------
     @property
@@ -625,20 +727,18 @@ class ObjectInspector(SpyderPluginWidget):
     def force_refresh(self):
         if self.source_is_console():
             self.set_object_text(None, force_refresh=True)
-        elif self._last_rope_data is not None:
-            text = self._last_rope_data
-            self.set_rope_doc(text, force_refresh=True)
+        elif self._last_editor_doc is not None:
+            self.set_editor_doc(self._last_editor_doc, force_refresh=True)
     
     def set_object_text(self, text, force_refresh=False, ignore_unknown=False):
         """Set object analyzed by Object Inspector"""
         if (self.locked and not force_refresh):
             return
-
         self.switch_to_console_source()
 
         add_to_combo = True
         if text is None:
-            text = unicode(self.combo.currentText())
+            text = to_text_string(self.combo.currentText())
             add_to_combo = False
             
         found = self.show_help(text, ignore_unknown=ignore_unknown)
@@ -647,7 +747,8 @@ class ObjectInspector(SpyderPluginWidget):
         
         if add_to_combo:
             self.combo.add_text(text)
-        self.save_history()
+        if found:
+            self.save_history()
         
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(True)
@@ -655,27 +756,25 @@ class ObjectInspector(SpyderPluginWidget):
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(False)
         
-    def set_rope_doc(self, text, force_refresh=False):
+    def set_editor_doc(self, doc, force_refresh=False):
         """
-        Use the object inspector to show text computed with rope from the
-        Editor plugin
+        Use the object inspector to show docstring dictionary computed
+        with introspection plugin from the Editor plugin
         """
         if (self.locked and not force_refresh):
             return
         self.switch_to_editor_source()
-        
-        self._last_rope_data = text
-        
-        self.object_edit.setText(text['obj_text'])
-        
+        self._last_editor_doc = doc
+        self.object_edit.setText(doc['obj_text'])
+
         if self.rich_help:
-            self.set_sphinx_text(text)
+            self.render_sphinx_doc(doc)
         else:
-            self.set_plain_text(text, is_code=False)
+            self.set_plain_text(doc, is_code=False)
         
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(True)
-        self.__eventually_raise_inspector(text['docstring'],
+        self.__eventually_raise_inspector(doc['docstring'],
                                           force=force_refresh)
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(False)
@@ -698,17 +797,17 @@ class ObjectInspector(SpyderPluginWidget):
     def load_history(self, obj=None):
         """Load history from a text file in user home directory"""
         if osp.isfile(self.LOG_PATH):
-            history = [line.replace('\n','')
-                       for line in file(self.LOG_PATH, 'r').readlines()]
+            history = [line.replace('\n', '')
+                       for line in open(self.LOG_PATH, 'r').readlines()]
         else:
             history = []
         return history
     
     def save_history(self):
         """Save history to a text file in user home directory"""
-        file(self.LOG_PATH, 'w').write("\n".join( \
-            [ unicode( self.combo.itemText(index) )
-                for index in range(self.combo.count()) ] ))
+        open(self.LOG_PATH, 'w').write("\n".join( \
+                [to_text_string(self.combo.itemText(index))
+                 for index in range(self.combo.count())] ))
         
     def toggle_plain_text(self, checked):
         """Toggle plain text docstring"""
@@ -731,7 +830,6 @@ class ObjectInspector(SpyderPluginWidget):
         if checked:
             self.docstring = not checked
             self.switch_to_rich_text()
-            self.force_refresh()
         self.set_option('rich_mode', checked)
         
     def toggle_auto_import(self, checked):
@@ -778,15 +876,10 @@ class ObjectInspector(SpyderPluginWidget):
                 self.shell = self.internal_shell
         return self.shell
         
-    def set_sphinx_text(self, text):
-        """Sphinxify text and display it"""
-        # If the thread is already running wait for it to finish before
-        # starting it again.
-        if self._sphinx_thread.wait():
-            # Math rendering option could have changed
-            self._sphinx_thread.math_option = self.get_option('math')
-            self._sphinx_thread.text = text
-            self._sphinx_thread.start()
+    def render_sphinx_doc(self, doc):
+        """Transform doc string dictionary to HTML and show it"""
+        # Math rendering option could have changed
+        self._sphinx_thread.render(doc, self.get_option('math'))
         
     def _on_sphinx_thread_html_ready(self, html_text):
         """Set our sphinx documentation based on thread result"""
@@ -808,7 +901,7 @@ class ObjectInspector(SpyderPluginWidget):
         shell = self.get_shell()
         if shell is None:
             return
-        obj_text = unicode(obj_text)
+        obj_text = to_text_string(obj_text)
         
         if not shell.is_defined(obj_text):
             if self.get_option('automatic_import') and\
@@ -816,26 +909,20 @@ class ObjectInspector(SpyderPluginWidget):
                 shell = self.internal_shell
             else:
                 shell = None
-                doc_text = None
+                doc = None
                 source_text = None
             
         if shell is not None:
-            doc_text = shell.get_doc(obj_text)
-            if isinstance(doc_text, bool):
-                doc_text = None
+            doc = shell.get_doc(obj_text)
             source_text = shell.get_source(obj_text)
             
         is_code = False
         
         if self.rich_help:
-            self.set_sphinx_text(doc_text)
-            if ignore_unknown:
-                return doc_text is not None
-            else:
-                return True
-        
+            self.render_sphinx_doc(doc)
+            return doc is not None
         elif self.docstring:
-            hlp_text = doc_text
+            hlp_text = doc
             if hlp_text is None:
                 hlp_text = source_text
                 if hlp_text is None:
@@ -845,13 +932,12 @@ class ObjectInspector(SpyderPluginWidget):
         else:
             hlp_text = source_text
             if hlp_text is None:
-                hlp_text = doc_text
+                hlp_text = doc
                 if hlp_text is None:
                     hlp_text = _("No source code available.")
                     if ignore_unknown:
                         return False
             else:
                 is_code = True
-        
         self.set_plain_text(hlp_text, is_code=is_code)
         return True
