@@ -27,6 +27,7 @@ from spyderlib.utils.qthelpers import get_icon, get_std_icon, create_action
 # Package local imports
 from spyderlib.widgets.comboboxes import PathComboBox
 from spyderlib.plugins import SpyderPluginMixin, PluginConfigPage
+from spyderlib.py3compat import to_text_string, getcwd
 
 
 class WorkingDirectoryConfigPage(PluginConfigPage):
@@ -56,7 +57,7 @@ class WorkingDirectoryConfigPage(PluginConfigPage):
                                         "directory will be the specified path"),
                                 button_group=startup_bg)
         thisdir_bd = self.create_browsedir("", 'startup/fixed_directory',
-                                           os.getcwdu())
+                                           getcwd())
         self.connect(thisdir_radio, SIGNAL("toggled(bool)"),
                      thisdir_bd.setEnabled)
         self.connect(lastdir_radio, SIGNAL("toggled(bool)"),
@@ -71,11 +72,11 @@ class WorkingDirectoryConfigPage(PluginConfigPage):
         editor_o_bg = QButtonGroup(editor_o_group)
         editor_o_radio1 = self.create_radiobutton(
                                 _("the current file directory"),
-                                'editor/open/browse_scriptdir', True,
+                                'editor/open/browse_scriptdir',
                                 button_group=editor_o_bg)
         editor_o_radio2 = self.create_radiobutton(
                                 _("the global working directory"),
-                                'editor/open/browse_workdir', False,
+                                'editor/open/browse_workdir', 
                                 button_group=editor_o_bg)
         
         editor_n_group = QGroupBox(_("New file"))
@@ -84,11 +85,11 @@ class WorkingDirectoryConfigPage(PluginConfigPage):
         editor_n_bg = QButtonGroup(editor_n_group)
         editor_n_radio1 = self.create_radiobutton(
                                 _("the current file directory"),
-                                'editor/new/browse_scriptdir', False,
+                                'editor/new/browse_scriptdir',
                                 button_group=editor_n_bg)
         editor_n_radio2 = self.create_radiobutton(
                                 _("the global working directory"),
-                                'editor/new/browse_workdir', True,
+                                'editor/new/browse_workdir',
                                 button_group=editor_n_bg)
         # Note: default values for the options above are set in plugin's
         #       constructor (see below)
@@ -96,9 +97,9 @@ class WorkingDirectoryConfigPage(PluginConfigPage):
         other_group = QGroupBox(_("Change to file base directory"))
         newcb = self.create_checkbox
         open_box = newcb(_("When opening a file"),
-                         'editor/open/auto_set_to_basedir', False)
+                         'editor/open/auto_set_to_basedir')
         save_box = newcb(_("When saving a file"),
-                         'editor/save/auto_set_to_basedir', False)
+                         'editor/save/auto_set_to_basedir')
         
         startup_layout = QVBoxLayout()
         startup_layout.addWidget(startup_label)
@@ -140,7 +141,7 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
     """
     CONF_SECTION = 'workingdir'
     CONFIGWIDGET_CLASS = WorkingDirectoryConfigPage
-    LOG_PATH = get_conf_path('.workingdir')
+    LOG_PATH = get_conf_path(CONF_SECTION)
     sig_option_changed = Signal(str, object)
     def __init__(self, parent, workdir=None):
         QToolBar.__init__(self, parent)
@@ -149,14 +150,6 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
         # Initialize plugin
         self.initialize_plugin()
         
-        # Setting default values for editor-related options
-        self.get_option('editor/open/browse_scriptdir', True)
-        self.get_option('editor/open/browse_workdir', False)
-        self.get_option('editor/new/browse_scriptdir', False)
-        self.get_option('editor/new/browse_workdir', True)
-        self.get_option('editor/open/auto_set_to_basedir', False)
-        self.get_option('editor/save/auto_set_to_basedir', False)
-        
         self.setWindowTitle(self.get_plugin_title()) # Toolbar title
         self.setObjectName(self.get_plugin_title()) # Used to save Window state
         
@@ -183,7 +176,7 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
                      self.next_action.setEnabled)
         
         # Path combo box
-        adjust = self.get_option('working_dir_adjusttocontents', False)
+        adjust = self.get_option('working_dir_adjusttocontents')
         self.pathedit = PathComboBox(self, adjust_to_contents=adjust)
         self.pathedit.setToolTip(_("This is the working directory for newly\n"
                                "opened consoles (Python interpreters and\n"
@@ -191,10 +184,10 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
                                "find in files plugin and for new files\n"
                                "created in the editor"))
         self.connect(self.pathedit, SIGNAL("open_dir(QString)"), self.chdir)
-        self.pathedit.setMaxCount(self.get_option('working_dir_history', 20))
+        self.pathedit.setMaxCount(self.get_option('working_dir_history'))
         wdhistory = self.load_wdhistory( workdir )
         if workdir is None:
-            if self.get_option('startup/use_last_directory', True):
+            if self.get_option('startup/use_last_directory'):
                 if wdhistory:
                     workdir = wdhistory[0]
                 else:
@@ -252,7 +245,7 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
         
     def refresh_plugin(self):
         """Refresh widget"""
-        curdir = os.getcwdu()
+        curdir = getcwd()
         self.pathedit.add_text(curdir)
         self.save_wdhistory()
         self.emit(SIGNAL("set_previous_enabled(bool)"),
@@ -277,13 +270,13 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
             wdhistory = [name for name in wdhistory if os.path.isdir(name)]
         else:
             if workdir is None:
-                workdir = os.getcwdu()
+                workdir = getcwd()
             wdhistory = [ workdir ]
         return wdhistory
     
     def save_wdhistory(self):
         """Save history to a text file in user home directory"""
-        text = [ unicode( self.pathedit.itemText(index) ) \
+        text = [ to_text_string( self.pathedit.itemText(index) ) \
                  for index in range(self.pathedit.count()) ]
         encoding.writelines(text, self.LOG_PATH)
         
@@ -291,7 +284,7 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
         """Select directory"""
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
         directory = getexistingdirectory(self.main, _("Select directory"),
-                                         os.getcwdu())
+                                         getcwd())
         if directory:
             self.chdir(directory)
         self.emit(SIGNAL('redirect_stdio(bool)'), True)
@@ -308,14 +301,14 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
         
     def parent_directory(self):
         """Change working directory to parent directory"""
-        self.chdir(os.path.join(os.getcwdu(), os.path.pardir))
+        self.chdir(os.path.join(getcwd(), os.path.pardir))
         
     def chdir(self, directory=None, browsing_history=False,
               refresh_explorer=True):
         """Set directory as working directory"""
         # Working directory history management
         if directory is not None:
-            directory = osp.abspath(unicode(directory))
+            directory = osp.abspath(to_text_string(directory))
         if browsing_history:
             directory = self.history[self.histindex]
         elif directory in self.history:
@@ -329,7 +322,7 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
             self.histindex = len(self.history)-1
         
         # Changing working directory
-        os.chdir( unicode(directory) )
+        os.chdir( to_text_string(directory) )
         self.refresh_plugin()
         if refresh_explorer:
             self.emit(SIGNAL("set_explorer_cwd(QString)"), directory)
@@ -337,4 +330,4 @@ class WorkingDirectory(QToolBar, SpyderPluginMixin):
         
     def set_as_current_console_wd(self):
         """Set as current console working directory"""
-        self.emit(SIGNAL("set_current_console_wd(QString)"), os.getcwdu())
+        self.emit(SIGNAL("set_current_console_wd(QString)"), getcwd())
