@@ -6,7 +6,7 @@
 
 """Running programs utilities"""
 
-from __future__ import with_statement
+from __future__ import print_function
 
 from distutils.version import LooseVersion
 import imp
@@ -18,7 +18,9 @@ import subprocess
 import sys
 import tempfile
 
+# Local imports
 from spyderlib.utils import encoding
+from spyderlib.py3compat import is_text_string
 
 
 if os.name == 'nt':
@@ -111,7 +113,7 @@ def shell_split(text):
     strings with spaces). This function is almost equivalent to the shlex.split
     function (see standard library `shlex`) except that it is supporting 
     unicode strings (shlex does not support unicode until Python 2.7.3)."""
-    assert isinstance(text, basestring)  # in case a QString is passed...
+    assert is_text_string(text)  # in case a QString is passed...
     pattern = r'(\s+|(?<!\\)".*?(?<!\\)"|(?<!\\)\'.*?(?<!\\)\')'
     out = []
     for token in re.split(pattern, text):
@@ -155,13 +157,13 @@ def run_python_script_in_terminal(fname, wdir, args, interact,
     p_args += get_python_args(fname, python_args, interact, debug, args)
     
     if os.name == 'nt':
-        cmd = 'start cmd.exe /c "cd %s && ' % wdir + ' '.join(p_args) + '"'
         # Command line and cwd have to be converted to the filesystem
         # encoding before passing them to subprocess
         # See http://bugs.python.org/issue1759845#msg74142
-        from spyderlib.utils.encoding import to_fs_from_unicode
-        subprocess.Popen(to_fs_from_unicode(cmd), shell=True,
-                         cwd=to_fs_from_unicode(wdir))
+        cmd = encoding.to_fs_from_unicode(
+                'start cmd.exe /c "cd %s && ' % wdir + ' '.join(p_args) + '"')
+        subprocess.Popen(cmd, shell=True,
+                         cwd=encoding.to_fs_from_unicode(wdir))
     elif os.name == 'posix':
         cmd = 'gnome-terminal'
         if is_program_installed(cmd):
@@ -192,6 +194,8 @@ def check_version(actver, version, cmp_op):
 
     Distributed under the terms of the BSD License.
     """
+    if isinstance(actver, tuple):
+        actver = '.'.join([str(i) for i in actver])
     try:
         if cmp_op == '>':
             return LooseVersion(actver) > LooseVersion(version)
@@ -201,6 +205,8 @@ def check_version(actver, version, cmp_op):
             return LooseVersion(actver) == LooseVersion(version)
         elif cmp_op == '<':
             return LooseVersion(actver) < LooseVersion(version)
+        elif cmp_op == '<=':
+            return LooseVersion(actver) <= LooseVersion(version)
         else:
             return False
     except TypeError:
@@ -242,17 +248,17 @@ def is_module_installed(module_name, version=None, installed_version=None,
                 f.write(get_modver + "\n")
                 f.write(ismod_inst + "\n")
                 if version:
-                    f.write("print is_module_installed('%s','%s')"\
+                    f.write("print(is_module_installed('%s','%s'))"\
                             % (module_name, version))
                 else:
-                    f.write("print is_module_installed('%s')" % module_name)
+                    f.write("print(is_module_installed('%s'))" % module_name)
             try:
-                output = subprocess.Popen([interpreter, script],
-                                      stdout=subprocess.PIPE).communicate()[0]
+                output, _err = subprocess.Popen([interpreter, script],
+                                        stdout=subprocess.PIPE).communicate()
             except subprocess.CalledProcessError:
-                output = 'True'
+                return True
             if output:  # TODO: Check why output could be empty!
-                return eval(output)
+                return eval(output.decode())
             else:
                 return False
         else:
@@ -284,15 +290,17 @@ def is_module_installed(module_name, version=None, installed_version=None,
             symb = version[:match.start()]
             if not symb:
                 symb = '='
-            assert symb in ('>=', '>', '=', '<'),\
+            assert symb in ('>=', '>', '=', '<', '<='),\
                     "Invalid version condition '%s'" % symb
             version = version[match.start():]
+            
             return check_version(actver, version, symb)
 
 
 if __name__ == '__main__':
-    print find_program('hg')
-    print shell_split('-q -o -a')
-    print shell_split(u'-q "d:\\Python de xxxx\\t.txt" -o -a')
-    print is_module_installed('IPython', '>=0.12')
-    print is_module_installed('IPython', '>=0.13;<1.0')
+    print(find_program('hg'))
+    print(shell_split('-q -o -a'))
+    print(shell_split('-q "d:\\Python de xxxx\\t.txt" -o -a'))
+    print(is_module_installed('IPython', '>=0.12'))
+    print(is_module_installed('IPython', '>=0.13;<1.0'))
+    print(is_module_installed('jedi', '>=0.7.0'))
