@@ -6,6 +6,8 @@
 
 """Editor tools: outline explorer, etc."""
 
+from __future__ import print_function
+
 import re
 import os.path as osp
 
@@ -19,6 +21,7 @@ from spyderlib.baseconfig import _, STDOUT
 from spyderlib.utils.qthelpers import (get_icon, create_action,
                                        create_toolbutton, set_item_user_text)
 from spyderlib.widgets.onecolumntree import OneColumnTree
+from spyderlib.py3compat import to_text_string
 
 
 #===============================================================================
@@ -78,7 +81,8 @@ class TreeItem(QTreeWidgetItem):
                 QTreeWidgetItem.__init__(self, parent, preceding,
                                          QTreeWidgetItem.Type)
         self.setText(0, name)
-        parent_text = from_qvariant(parent.data(0, Qt.UserRole), unicode)
+        parent_text = from_qvariant(parent.data(0, Qt.UserRole),
+                                    to_text_string)
         set_item_user_text(self, parent_text+'/'+name)
         self.line = line
         
@@ -100,7 +104,7 @@ class FunctionItem(TreeItem):
     def setup(self):
         if self.is_method():
             self.setToolTip(0, _("Method defined at line %s") % str(self.line))
-            name = unicode(self.text(0))
+            name = to_text_string(self.text(0))
             if name.startswith('__'):
                 self.set_icon('private2.png')
             elif name.startswith('_'):
@@ -112,6 +116,36 @@ class FunctionItem(TreeItem):
             self.setToolTip(0, _("Function defined at line %s"
                                  ) % str(self.line))
 
+class CommentItem(TreeItem):
+    def __init__(self, name, line, parent, preceding):
+        name = name.lstrip("# ")
+        TreeItem.__init__(self, name, line, parent, preceding)
+
+    def setup(self):
+        self.set_icon('blockcomment.png')
+        font = self.font(0)
+        font.setItalic(True)
+        self.setFont(0, font)
+        self.setToolTip(0, _("Line %s") % str(self.line))
+
+class CellItem(TreeItem):
+    def __init__(self, name, line, parent, preceding):
+        name = name.lstrip("#% ")
+        if name.startswith("<codecell>"):
+            name = name[10:].lstrip()
+        elif name.startswith("In["):
+            name = name[2:]
+            if name.endswith("]:"):
+                name = name[:-1]
+            name = name.strip()
+        TreeItem.__init__(self, name, line, parent, preceding)
+
+    def setup(self):
+        self.set_icon('cell.png')
+        font = self.font(0)
+        font.setItalic(True)
+        self.setFont(0, font)
+        self.setToolTip(0, _("Cell starts at line %s") % str(self.line))
 
 def get_item_children(item):
     children = [item.child(index) for index in range(item.childCount())]
@@ -131,7 +165,7 @@ def item_at_line(root_item, line):
 
 def remove_from_tree_cache(tree_cache, line=None, item=None):
     if line is None:
-        for line, (_it, _level, _debug) in tree_cache.iteritems():
+        for line, (_it, _level, _debug) in list(tree_cache.items()):
             if _it is item:
                 break
     item, _level, debug = tree_cache.pop(line)
@@ -142,7 +176,7 @@ def remove_from_tree_cache(tree_cache, line=None, item=None):
     except RuntimeError:
         # Item has already been deleted
         #XXX: remove this debug-related fragment of code
-        print >>STDOUT, "unable to remove tree item: ", debug
+        print("unable to remove tree item: ", debug, file=STDOUT)
 
 class OutlineExplorerTreeWidget(OneColumnTree):
     def __init__(self, parent, show_fullpath=False, fullpath_sorting=True,
@@ -225,7 +259,7 @@ class OutlineExplorerTreeWidget(OneColumnTree):
     def set_current_editor(self, editor, fname, update):
         """Bind editor instance"""
         editor_id = editor.get_document_id()
-        if editor_id in self.editor_ids.values():
+        if editor_id in list(self.editor_ids.values()):
             item = self.editor_items[editor_id]
             if not self.freeze:
                 self.scrollToItem(item)
@@ -256,14 +290,14 @@ class OutlineExplorerTreeWidget(OneColumnTree):
     def file_renamed(self, editor, new_filename):
         """File was renamed, updating outline explorer tree"""
         editor_id = editor.get_document_id()
-        if editor_id in self.editor_ids.values():
+        if editor_id in list(self.editor_ids.values()):
             root_item = self.editor_items[editor_id]
             root_item.set_path(new_filename, fullpath=self.show_fullpath)
             self.__sort_toplevel_items()
         
     def update_all(self):
         self.save_expanded_state()
-        for editor, editor_id in self.editor_ids.iteritems():
+        for editor, editor_id in list(self.editor_ids.items()):
             item = self.editor_items[editor_id]
             tree_cache = self.editor_tree_cache[editor_id]
             self.populate_branch(editor, item, tree_cache)
@@ -274,7 +308,7 @@ class OutlineExplorerTreeWidget(OneColumnTree):
             if self.current_editor is editor:
                 self.current_editor = None
             editor_id = self.editor_ids.pop(editor)
-            if editor_id not in self.editor_ids.values():
+            if editor_id not in list(self.editor_ids.values()):
                 root_item = self.editor_items.pop(editor_id)
                 self.editor_tree_cache.pop(editor_id)
                 try:
@@ -295,7 +329,7 @@ class OutlineExplorerTreeWidget(OneColumnTree):
             tree_cache = {}
         
         # Removing cached items for which line is > total line nb
-        for _l in tree_cache.keys():
+        for _l in list(tree_cache.keys()):
             if _l >= editor.get_line_count():
                 # Checking if key is still in tree cache in case one of its 
                 # ancestors was deleted in the meantime (deleting all children):
@@ -307,7 +341,7 @@ class OutlineExplorerTreeWidget(OneColumnTree):
         previous_level = None
         
         oe_data = editor.highlighter.get_outlineexplorer_data()
-        
+        editor.has_cell_separators = oe_data.get('found_cell_separators', False)
         for block_nb in range(editor.get_line_count()):
             line_nb = block_nb+1
             data = oe_data.get(block_nb)
@@ -348,7 +382,7 @@ class OutlineExplorerTreeWidget(OneColumnTree):
             parent, _level = ancestors[-1]
             
             if citem is not None:
-                cname = unicode(citem.text(0))
+                cname = to_text_string(citem.text(0))
                 
             preceding = root_item if previous_item is None else previous_item
             if not_class_nor_function:
@@ -363,7 +397,14 @@ class OutlineExplorerTreeWidget(OneColumnTree):
                         continue
                     else:
                         remove_from_tree_cache(tree_cache, line=line_nb)
-                item = TreeItem(data.text, line_nb, parent, preceding)
+                if data.is_comment():
+                    if data.def_type == data.CELL:
+                        item = CellItem(data.text, line_nb, parent, preceding)
+                    else:
+                        item = CommentItem(
+                            data.text, line_nb, parent, preceding)
+                else:
+                    item = TreeItem(data.text, line_nb, parent, preceding)
             elif class_name is not None:
                 if citem is not None:
                     if class_name == cname and level == clevel:
@@ -385,8 +426,8 @@ class OutlineExplorerTreeWidget(OneColumnTree):
                 
             item.setup()
             debug = "%s -- %s/%s" % (str(item.line).rjust(6),
-                                     unicode(item.parent().text(0)),
-                                     unicode(item.text(0)))
+                                     to_text_string(item.parent().text(0)),
+                                     to_text_string(item.text(0)))
             tree_cache[line_nb] = (item, level, debug)
             previous_level = level
             previous_item = item
@@ -429,10 +470,10 @@ class OutlineExplorerTreeWidget(OneColumnTree):
             self.parent().emit(SIGNAL("edit(QString)"), root_item.path)
         self.freeze = False
         parent = self.current_editor.parent()
-        for editor_id, i_item in self.editor_items.iteritems():
+        for editor_id, i_item in list(self.editor_items.items()):
             if i_item is root_item:
                 #XXX: not working anymore!!!
-                for editor, _id in self.editor_ids.iteritems():
+                for editor, _id in list(self.editor_ids.items()):
                     if _id == editor_id and editor.parent() is parent:
                         self.current_editor = editor
                         break
@@ -470,14 +511,14 @@ class OutlineExplorerWidget(QWidget):
         self.visibility_action.setChecked(True)
         
         btn_layout = QHBoxLayout()
-        btn_layout.setAlignment(Qt.AlignRight)
+        btn_layout.setAlignment(Qt.AlignLeft)
         for btn in self.setup_buttons():
             btn_layout.addWidget(btn)
 
         layout = QVBoxLayout()
         layout.setContentsMargins(0, 0, 0, 0)
-        layout.addWidget(self.treewidget)
         layout.addLayout(btn_layout)
+        layout.addWidget(self.treewidget)
         self.setLayout(layout)
         
     def toggle_visibility(self, state):
