@@ -6,9 +6,47 @@ import os
 import os.path as osp
 import pdb
 import bdb
-import __builtin__
 
 
+# sys.argv can be missing when Python is embedded, taking care of it.
+# Fixes Issue 1473 and other crazy crashes with IPython 0.13 trying to
+# access it.
+if not hasattr(sys, 'argv'):
+    sys.argv = ['']
+
+#==============================================================================
+# Important Note:
+#
+# We avoid importing spyderlib here, so we are handling Python 3 compatiblity
+# by hand.
+#==============================================================================
+def _print(*objects, **options):
+    end = options.get('end', '\n')
+    file = options.get('file', sys.stdout)
+    sep = options.get('sep', ' ')
+    string = sep.join([str(obj) for obj in objects])
+    if sys.version[0] == '3':
+        # Python 3
+        local_dict = {}
+        exec('printf = print', local_dict) # to avoid syntax error in Python 2
+        local_dict['printf'](string, file=file, end=end, sep=sep)
+    else:
+        # Python 2
+        if end:
+            print >>file, string
+        else:
+            print >>file, string,
+
+try:
+    import __builtin__ as builtins
+except ImportError:
+    # Python 3
+    import builtins
+    basestring = (str,)
+    def execfile(filename, namespace):
+        # Open a source file correctly, whatever its encoding is
+        exec(compile(open(filename, 'rb').read(), filename, 'exec'), namespace)
+
 # Colorization of sys.stderr (standard Python interpreter)
 if os.environ.get("COLORIZE_SYS_STDERR", "").lower() == "true":
     class StderrProxy(object):
@@ -100,8 +138,7 @@ if sys.platform == 'darwin' and 'Spyder.app' in __file__:
         # Add a minimal library (with spyderlib) at the end of sys.path to
         # be able to connect our monitor to the external console
         app_pythonpath = 'Spyder.app/Contents/Resources/lib/python2.7'
-        full_pythonpath = filter(lambda p: p.endswith(app_pythonpath),
-                                 sys.path)
+        full_pythonpath = [p for p in sys.path if p.endswith(app_pythonpath)]
         if full_pythonpath:
             sys.path.remove(full_pythonpath[0])
             sys.path.append(full_pythonpath[0] + osp.sep + 'minimal-lib')
@@ -127,6 +164,7 @@ if mpl_backend:
             if mpl_backend == 'Qt4Agg' and sys.platform == 'darwin':
                 mpl_backend = 'MacOSX'
         matplotlib.rcParams['docstring.hardcopy'] = True
+        matplotlib.rcParams['interactive'] = True
         matplotlib.use(mpl_backend)
     except ImportError:
         pass
@@ -141,7 +179,7 @@ if os.environ.get("MATPLOTLIB_PATCH", "").lower() == "true":
 
 
 # Set standard outputs encoding:
-# (otherwise, for example, print u"é" will fail)
+# (otherwise, for example, print("é") will fail)
 encoding = None
 try:
     import locale
@@ -155,8 +193,12 @@ else:
 if encoding is None:
     encoding = "UTF-8"
 
-sys.setdefaultencoding(encoding)
-os.environ['SPYDER_ENCODING'] = encoding
+try:
+    sys.setdefaultencoding(encoding)
+    os.environ['SPYDER_ENCODING'] = encoding
+except AttributeError:
+    # Python 3
+    pass
     
 try:
     import sitecustomize  #analysis:ignore
@@ -192,16 +234,16 @@ else:
             try:
                 source = source.__file__
             except AttributeError:
-                print >>sys.stderr, "The argument must be either a string"\
-                                    "or a module object"
+                raise ValueError("source argument must be either "
+                                 "a string or a module object")
         if source.endswith('.pyc'):
             source = source[:-1]
         source = osp.abspath(source)
         if osp.exists(source):
             monitor.notify_open_file(source, lineno=lineno)
         else:
-            print >>sys.stderr, "Can't open file %s" % source
-    __builtin__.open_in_spyder = open_in_spyder
+            _print("Can't open file %s" % source, file=sys.stderr)
+    builtins.open_in_spyder = open_in_spyder
     
     # * PyQt4:
     #   * Removing PyQt4 input hook which is not working well on Windows since 
@@ -325,7 +367,7 @@ class SpyderPdb(pdb.Pdb):
         if CONF.get('run', 'breakpoints/enabled', True):
             breakpoints = CONF.get('run', 'breakpoints', {})
             i = 0
-            for fname, data in breakpoints.iteritems():
+            for fname, data in list(breakpoints.items()):
                 for linenumber, condition in data:
                     i += 1
                     self.set_break(self.canonic(fname), linenumber,
@@ -364,7 +406,7 @@ def monkeypatch_method(cls, patch_name):
         if old_func is not None:
             # Add the old func to a list of old funcs.
             old_ref = "_old_%s_%s" % (patch_name, fname)
-            #print old_ref, old_func
+            #print(old_ref, old_func)
             old_attr = getattr(cls, old_ref, None)
             if old_attr is None:
                 setattr(cls, old_ref, old_func)
@@ -429,8 +471,8 @@ if os.environ.get("IGNORE_SIP_SETAPI_ERRORS", "").lower() == "true":
         def patched_setapi(name, no):
             try:
                 original_setapi(name, no)
-            except ValueError, msg:
-                print >>sys.stderr, "Warning/PyQt4-Spyder (%s)" % str(msg)
+            except ValueError as msg:
+                _print("Warning/PyQt4-Spyder (%s)" % str(msg), file=sys.stderr)
         sip.setapi = patched_setapi
     except ImportError:
         pass
@@ -453,7 +495,7 @@ class UserModuleDeleter(object):
         if pathlist is None:
             pathlist = []
         self.pathlist = pathlist
-        self.previous_modules = sys.modules.keys()
+        self.previous_modules = list(sys.modules.keys())
 
     def is_module_blacklisted(self, modname, modpath):
         for path in [sys.prefix]+self.pathlist:
@@ -471,7 +513,7 @@ class UserModuleDeleter(object):
         Do not del C modules
         """
         log = []
-        for modname, module in sys.modules.items():
+        for modname, module in list(sys.modules.items()):
             if modname not in self.previous_modules:
                 modpath = getattr(module, '__file__', None)
                 if modpath is None:
@@ -483,8 +525,8 @@ class UserModuleDeleter(object):
                     log.append(modname)
                     del sys.modules[modname]
         if verbose and log:
-            print "\x1b[4;33m%s\x1b[24m%s\x1b[0m"\
-                  % ("UMD has deleted", ": "+", ".join(log))
+            _print("\x1b[4;33m%s\x1b[24m%s\x1b[0m"\
+                   % ("UMD has deleted", ": "+", ".join(log)))
 
 __umd__ = None
 
@@ -510,7 +552,9 @@ def runfile(filename, args=None, wdir=None, namespace=None):
     """
     try:
         filename = filename.decode('utf-8')
-    except (UnicodeError, TypeError):
+    except (UnicodeError, TypeError, AttributeError):
+        # UnicodeError, TypeError --> eventually raised in Python 2
+        # AttributeError --> systematically raised in Python 3
         pass
     global __umd__
     if os.environ.get("UMD_ENABLED", "").lower() == "true":
@@ -534,14 +578,16 @@ def runfile(filename, args=None, wdir=None, namespace=None):
     if wdir is not None:
         try:
             wdir = wdir.decode('utf-8')
-        except (UnicodeError, TypeError):
+        except (UnicodeError, TypeError, AttributeError):
+            # UnicodeError, TypeError --> eventually raised in Python 2
+            # AttributeError --> systematically raised in Python 3
             pass
         os.chdir(wdir)
     execfile(filename, namespace)
     sys.argv = ['']
     namespace.pop('__file__')
     
-__builtin__.runfile = runfile
+builtins.runfile = runfile
 
 
 def debugfile(filename, args=None, wdir=None):
@@ -559,7 +605,7 @@ def debugfile(filename, args=None, wdir=None):
         filename = filename.replace('\\', '/')
     debugger.run("runfile(%r, args=%r, wdir=%r)" % (filename, args, wdir))
 
-__builtin__.debugfile = debugfile
+builtins.debugfile = debugfile
 
 
 def evalsc(command):
@@ -575,7 +621,7 @@ def evalsc(command):
         else:
             from subprocess import Popen, PIPE
             Popen(command, shell=True, stdin=PIPE)
-            print '\n'
+            _print('\n')
     else:
         # General command
         namespace = _get_globals()
@@ -592,7 +638,10 @@ def evalsc(command):
                 except KeyError:
                     pass
         elif command in ('cd', 'pwd'):
-            print os.getcwdu()
+            try:
+                _print(os.getcwdu())
+            except AttributeError:
+                _print(os.getcwd())
         elif command == 'ls':
             if os.name == 'nt':
                 evalsc('!dir')
@@ -604,7 +653,7 @@ def evalsc(command):
         else:
             raise NotImplementedError("Unsupported command: '%s'" % command)
 
-__builtin__.evalsc = evalsc
+builtins.evalsc = evalsc
 
 
 # Restoring original PYTHONPATH
