@@ -14,16 +14,20 @@ IPython console plugin.
 import os
 import re
 import sre_constants
+import textwrap
+from xml.sax.saxutils import escape
 
 from spyderlib.qt.QtGui import (QTextCursor, QTextDocument, QApplication,
-                                QCursor)
-from spyderlib.qt.QtCore import Qt, QRegExp, SIGNAL
+                                QCursor, QToolTip)
+from spyderlib.qt.QtCore import Qt, QPoint, QRegExp, SIGNAL
 
 # Local imports
 from spyderlib.baseconfig import _
 from spyderlib.utils import encoding, sourcecode
 from spyderlib.utils.misc import get_error_match
-from spyderlib.utils.dochelpers import getobj
+from spyderlib.utils.dochelpers import (getobj, getargspecfromtext,
+                                        getsignaturefromtext)
+from spyderlib.py3compat import is_text_string, to_text_string, u
 
 
 HISTORY_FILENAMES = []
@@ -32,12 +36,85 @@ HISTORY_FILENAMES = []
 class BaseEditMixin(object):
     def __init__(self):
         self.eol_chars = None
-        
+        self.calltip_size = 600
+    
+    
+    #------Line number area
+    def get_linenumberarea_width(self):
+        """Return line number area width"""
+        # Implemented in CodeEditor, but needed for calltip/completion widgets
+        return 0
+    
+    
+    #------Calltips
+    def _format_signature(self, text):
+        formatted_lines = []
+        name = text.split('(')[0]
+        rows = textwrap.wrap(text, width=50,
+                             subsequent_indent=' '*(len(name)+1))
+        for r in rows:
+            r = escape(r) # Escape most common html chars
+            r = r.replace(' ', '&nbsp;')
+            for char in ['=', ',', '(', ')', '*', '**']:
+                r = r.replace(char,
+                       '<span style=\'color: red; font-weight: bold\'>' + \
+                       char + '</span>')
+            formatted_lines.append(r)
+        signature = '<br>'.join(formatted_lines)
+        return signature, rows
+    
+    def show_calltip(self, title, text, signature=False, color='#2D62FF',
+                     at_line=None, at_position=None):
+        """Show calltip"""
+        if text is None or len(text) == 0:
+            return
+
+        # Saving cursor position:
+        if at_position is None:
+            at_position = self.get_position('cursor')
+        self.calltip_position = at_position
+
+        # Preparing text:
+        if signature:
+            text, wrapped_textlines = self._format_signature(text)
+        else:
+            if isinstance(text, list):
+                text = "\n    ".join(text)
+            text = text.replace('\n', '<br>')
+            if len(text) > self.calltip_size:
+                text = text[:self.calltip_size] + " ..."
+
+        # Formatting text
+        font = self.font()
+        size = font.pointSize()
+        family = font.family()
+        format1 = '<div style=\'font-family: "%s"; font-size: %spt; color: %s\'>'\
+                  % (family, size, color)
+        format2 = '<div style=\'font-family: "%s"; font-size: %spt\'>'\
+                  % (family, size-1 if size > 9 else size)
+        tiptext = format1 + ('<b>%s</b></div>' % title) + '<hr>' + \
+                  format2 + text + "</div>"
+
+        # Showing tooltip at cursor position:
+        cx, cy = self.get_coordinates('cursor')
+        if at_line is not None:
+            cx = 5
+            cursor = QTextCursor(self.document().findBlockByNumber(at_line-1))
+            cy = self.cursorRect(cursor).top()
+        point = self.mapToGlobal(QPoint(cx, cy))
+        point.setX(point.x()+self.get_linenumberarea_width())
+        point.setY(point.y()+font.pointSize()+5)
+        if signature:
+            self.calltip_widget.show_tip(point, tiptext, wrapped_textlines)
+        else:
+            QToolTip.showText(point, tiptext)
+    
+    
     #------EOL characters
     def set_eol_chars(self, text):
         """Set widget end-of-line (EOL) characters from text (analyzes text)"""
-        if not isinstance(text, basestring): # testing for QString (PyQt API#1)
-            text = unicode(text)
+        if not is_text_string(text): # testing for QString (PyQt API#1)
+            text = to_text_string(text)
         eol_chars = sourcecode.get_eol_chars(text)
         if eol_chars is not None and self.eol_chars is not None:
             self.document().setModified(True)
@@ -53,7 +130,7 @@ class BaseEditMixin(object):
     def get_text_with_eol(self):
         """Same as 'toPlainText', replace '\n' 
         by correct end-of-line characters"""
-        utext = unicode(self.toPlainText())
+        utext = to_text_string(self.toPlainText())
         lines = utext.splitlines()
         linesep = self.get_line_separator()
         txt = linesep.join(lines)
@@ -193,7 +270,7 @@ class BaseEditMixin(object):
         # since splitlines doesn't return that line as the last element
         # TODO: Make this function more efficient
         try:
-            return unicode(self.toPlainText()).splitlines()[line_nb]
+            return to_text_string(self.toPlainText()).splitlines()[line_nb]
         except IndexError:
             return self.get_line_separator()
     
@@ -203,11 +280,11 @@ class BaseEditMixin(object):
         Positions may be positions or 'sol', 'eol', 'sof', 'eof' or 'cursor'
         """
         cursor = self.__select_text(position_from, position_to)
-        text = unicode(cursor.selectedText())
+        text = to_text_string(cursor.selectedText())
         if text:
             while text.endswith("\n"):
                 text = text[:-1]
-            while text.endswith(u"\u2029"):
+            while text.endswith(u("\u2029")):
                 text = text[:-1]
         return text
     
@@ -220,7 +297,7 @@ class BaseEditMixin(object):
             cursor.setPosition(position)
             cursor.movePosition(QTextCursor.Right,
                                 QTextCursor.KeepAnchor)
-            return unicode(cursor.selectedText())
+            return to_text_string(cursor.selectedText())
         else:
             return ''
     
@@ -259,14 +336,14 @@ class BaseEditMixin(object):
             def is_space(move):
                 curs = self.textCursor()
                 curs.movePosition(move, QTextCursor.KeepAnchor)
-                return not unicode(curs.selectedText()).strip()
+                return not to_text_string(curs.selectedText()).strip()
             if is_space(QTextCursor.NextCharacter):
                 if is_space(QTextCursor.PreviousCharacter):
                     return
                 cursor.movePosition(QTextCursor.WordLeft)
 
         cursor.select(QTextCursor.WordUnderCursor)
-        text = unicode(cursor.selectedText())
+        text = to_text_string(cursor.selectedText())
         match = re.findall(r'([a-zA-Z\_]+[0-9a-zA-Z\_]*)', text)
         if match:
             return match[0]
@@ -275,7 +352,7 @@ class BaseEditMixin(object):
         """Return current line's text"""
         cursor = self.textCursor()
         cursor.select(QTextCursor.BlockUnderCursor)
-        return unicode(cursor.selectedText())
+        return to_text_string(cursor.selectedText())
     
     def get_current_line_to_cursor(self):
         """Return text from prompt to cursor"""
@@ -290,17 +367,17 @@ class BaseEditMixin(object):
         """Return line at *coordinates* (QPoint)"""
         cursor = self.cursorForPosition(coordinates)
         cursor.select(QTextCursor.BlockUnderCursor)
-        return unicode(cursor.selectedText()).replace(u'\u2029', '')
+        return to_text_string(cursor.selectedText()).replace(u('\u2029'), '')
     
     def get_word_at(self, coordinates):
         """Return word at *coordinates* (QPoint)"""
         cursor = self.cursorForPosition(coordinates)
         cursor.select(QTextCursor.WordUnderCursor)
-        return unicode(cursor.selectedText())
+        return to_text_string(cursor.selectedText())
     
     def get_block_indentation(self, block_nb):
         """Return line indentation (character number)"""
-        text = unicode(self.document().findBlockByNumber(block_nb).text())
+        text = to_text_string(self.document().findBlockByNumber(block_nb).text())
         return len(text)-len(text.lstrip())
     
     def get_selection_bounds(self):
@@ -315,7 +392,7 @@ class BaseEditMixin(object):
     #------Text selection
     def has_selected_text(self):
         """Returns True if some text is selected"""
-        return bool(unicode(self.textCursor().selectedText()))
+        return bool(to_text_string(self.textCursor().selectedText()))
 
     def get_selected_text(self):
         """
@@ -324,7 +401,7 @@ class BaseEditMixin(object):
         Replace the unicode line separator character \u2029 by 
         the line separator characters returned by get_line_separator
         """
-        return unicode(self.textCursor().selectedText()).replace(u"\u2029",
+        return to_text_string(self.textCursor().selectedText()).replace(u("\u2029"),
                                                      self.get_line_separator())
     
     def remove_selected_text(self):
@@ -338,10 +415,11 @@ class BaseEditMixin(object):
         cursor = self.textCursor()
         cursor.beginEditBlock()
         if pattern is not None:
-            seltxt = unicode(cursor.selectedText())
+            seltxt = to_text_string(cursor.selectedText())
         cursor.removeSelectedText()
         if pattern is not None:
-            text = re.sub(unicode(pattern), unicode(text), unicode(seltxt))
+            text = re.sub(to_text_string(pattern),
+                          to_text_string(text), to_text_string(seltxt))
         cursor.insertText(text)
         cursor.endEditBlock()
 
@@ -351,8 +429,8 @@ class BaseEditMixin(object):
         """Reimplement QTextDocument's find method
         
         Add support for *multiline* regular expressions"""
-        pattern = unicode(regexp.pattern())
-        text = unicode(self.toPlainText())
+        pattern = to_text_string(regexp.pattern())
+        text = to_text_string(self.toPlainText())
         try:
             regobj = re.compile(pattern)
         except sre_constants.error:
@@ -388,7 +466,7 @@ class BaseEditMixin(object):
         if forward:
             moves += [QTextCursor.NextWord, QTextCursor.Start]
             if changed:
-                if unicode(cursor.selectedText()):
+                if to_text_string(cursor.selectedText()):
                     new_position = min([cursor.selectionStart(),
                                         cursor.selectionEnd()])
                     cursor.setPosition(new_position)
@@ -397,7 +475,7 @@ class BaseEditMixin(object):
         else:
             moves += [QTextCursor.End]
         if not regexp:
-            text = re.escape(unicode(text))
+            text = re.escape(to_text_string(text))
         pattern = QRegExp(r"\b%s\b" % text if words else text,
                           Qt.CaseSensitive if case else Qt.CaseInsensitive,
                           QRegExp.RegExp2)
@@ -457,7 +535,7 @@ class TracebackLinksMixin(object):
 class InspectObjectMixin(object):
     def __init__(self):
         self.inspector = None
-        self.inspector_enabled = True
+        self.inspector_enabled = False
     
     def set_inspector(self, inspector):
         """Set ObjectInspector DockWidget reference"""
@@ -478,12 +556,13 @@ class InspectObjectMixin(object):
         if tl2 and text2.startswith(tl2[0]):
             text += tl2[0]
         if text:
-            self.show_docstring(text, force=True)
+            self.show_object_info(text, force=True)
     
-    def show_docstring(self, text, call=False, force=False):
-        """Show docstring or arguments"""
-        text = unicode(text) # Useful only for ExternalShellBase
+    def show_object_info(self, text, call=False, force=False):
+        """Show signature calltip and/or docstring in the Object Inspector"""
+        text = to_text_string(text) # Useful only for ExternalShellBase
         
+        # Show docstring
         insp_enabled = self.inspector_enabled or force
         if force and self.inspector is not None:
             self.inspector.dockwidget.setVisible(True)
@@ -492,26 +571,41 @@ class InspectObjectMixin(object):
            (self.inspector.dockwidget.isVisible()):
             # ObjectInspector widget exists and is visible
             self.inspector.set_shell(self)
-            self.inspector.set_object_text(text, ignore_unknown=True)
+            self.inspector.set_object_text(text, ignore_unknown=False)
             self.setFocus() # if inspector was not at top level, raising it to
                             # top will automatically give it focus because of
                             # the visibility_changed signal, so we must give
                             # focus back to shell
-            if call and self.calltips:
-                # Display argument list if this is function call
-                iscallable = self.iscallable(text)
-                if iscallable is not None:
-                    if iscallable:
-                        arglist = self.get_arglist(text)
-                        if isinstance(arglist, bool):
-                            arglist = []
-                        if arglist:
-                            self.show_calltip(_("Arguments"),
-                                              arglist, '#129625')
-        elif self.calltips: # inspector is not visible or link is disabled
-            doc = self.get__doc__(text)
-            if doc is not None:
-                self.show_calltip(_("Documentation"), doc)
+        
+        # Show calltip
+        if call and self.calltips:
+            # Display argument list if this is a function call
+            iscallable = self.iscallable(text)
+            if iscallable is not None:
+                if iscallable:
+                    arglist = self.get_arglist(text)
+                    name =  text.split('.')[-1]
+                    argspec = signature = ''
+                    if isinstance(arglist, bool):
+                        arglist = []
+                    if arglist:
+                        argspec = '(' + ''.join(arglist) + ')'
+                    else:
+                        doc = self.get__doc__(text)
+                        if doc is not None:
+                            # This covers cases like np.abs, whose docstring is
+                            # the same as np.absolute and because of that a
+                            # proper signature can't be obtained correctly
+                            argspec = getargspecfromtext(doc)
+                            if not argspec:
+                                signature = getsignaturefromtext(doc, name)
+                    if argspec or signature:
+                        if argspec:
+                            tiptext = name + argspec
+                        else:
+                            tiptext = signature
+                        self.show_calltip(_("Arguments"), tiptext,
+                                          signature=True, color='#2D62FF')
     
     def get_last_obj(self, last=False):
         """
@@ -530,7 +624,7 @@ class SaveHistoryMixin(object):
     
     def add_to_history(self, command):
         """Add command to history"""
-        command = unicode(command)
+        command = to_text_string(command)
         if command in ['', '\n'] or command.startswith('Traceback'):
             return
         if command.endswith('\n'):
