@@ -13,19 +13,24 @@
 
 import os
 import re
-import string
 import sys
 
 from spyderlib.qt.QtGui import (QTextCursor, QColor, QFont, QApplication,
                                 QTextEdit, QTextCharFormat, QToolTip,
                                 QListWidget, QPlainTextEdit, QPalette,
-                                QMainWindow, QTextOption, QMouseEvent)
-from spyderlib.qt.QtCore import QPoint, SIGNAL, Qt, QEventLoop, QEvent
+                                QMainWindow, QTextOption, QMouseEvent,
+                                QTextFormat)
+from spyderlib.qt.QtCore import SIGNAL, Qt, QEventLoop, QEvent, QPoint
+from spyderlib.qt.compat import to_qvariant
 
 
 # Local imports
 from spyderlib.widgets.sourcecode.terminal import ANSIEscapeCodeHandler
 from spyderlib.widgets.mixins import BaseEditMixin
+from spyderlib.widgets.calltip import CallTipWidget
+from spyderlib.py3compat import to_text_string, str_lower
+
+CELL_SEPARATORS = ('#%%', '# %%', '# <codecell>', '# In[')
 
 
 class CompletionWidget(QListWidget):
@@ -36,7 +41,6 @@ class CompletionWidget(QListWidget):
         self.textedit = parent
         self.completion_list = None
         self.case_sensitive = False
-        self.show_single = None
         self.enter_select = None
         self.hide()
         self.connect(self, SIGNAL("itemActivated(QListWidgetItem*)"),
@@ -47,7 +51,7 @@ class CompletionWidget(QListWidget):
         self.setFont(font)
         
     def show_list(self, completion_list, automatic=True):
-        if not self.show_single and len(completion_list) == 1 and not automatic:
+        if len(completion_list) == 1 and not automatic:
             self.textedit.insert_completion(completion_list[0])
             return
         
@@ -105,7 +109,7 @@ class CompletionWidget(QListWidget):
             point = ancestor.mapFromGlobal(point)
         self.move(point)
         
-        if unicode(self.textedit.completion_text):
+        if to_text_string(self.textedit.completion_text):
             # When initialized, if completion text is not empty, we need 
             # to update the displayed list:
             self.update_current()
@@ -141,7 +145,7 @@ class CompletionWidget(QListWidget):
             QListWidget.keyPressEvent(self, event)
             
     def update_current(self):
-        completion_text = unicode(self.textedit.completion_text)
+        completion_text = to_text_string(self.textedit.completion_text)
         if completion_text:
             for row, completion in enumerate(self.completion_list):
                 if not self.case_sensitive:
@@ -169,14 +173,13 @@ class CompletionWidget(QListWidget):
     def item_selected(self, item=None):
         if item is None:
             item = self.currentItem()
-        self.textedit.insert_completion( unicode(item.text()) )
+        self.textedit.insert_completion( to_text_string(item.text()) )
         self.hide()
 
 
 class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     """Text edit base widget"""
     BRACE_MATCHING_SCOPE = ('sof', 'eof')
-    CELL_SEPARATORS = ('#%%', '# %%', '# <codecell>')
     
     def __init__(self, parent=None):
         QPlainTextEdit.__init__(self, parent)
@@ -203,23 +206,23 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.completion_widget = CompletionWidget(self, parent)
         self.codecompletion_auto = False
         self.codecompletion_case = True
-        self.codecompletion_single = False
         self.codecompletion_enter = False
         self.calltips = True
         self.calltip_position = None
-        self.calltip_size = 600
-        self.calltip_font = QFont()
+        self.has_cell_separators = False        
         self.completion_text = ""
+        self.calltip_widget = CallTipWidget(self, hide_timer_on=True)
+        
+        # The color values may be overridden by the syntax highlighter 
+        # Highlight current line color
+        self.currentline_color = QColor(Qt.red).lighter(190)
+        self.currentcell_color = QColor(Qt.red).lighter(194)
 
         # Brace matching
         self.bracepos = None
         self.matched_p_color = QColor(Qt.green)
         self.unmatched_p_color = QColor(Qt.red)
         
-    def setup_calltips(self, size=None, font=None):
-        self.calltip_size = size
-        self.calltip_font = font
-    
     def setup_completion(self, size=None, font=None):
         self.completion_widget.setup_appearance(size, font)
         
@@ -235,13 +238,8 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         palette.setColor(QPalette.Base, background)
         palette.setColor(QPalette.Text, foreground)
         self.setPalette(palette)
-        
-    #------Line number area
-    def get_linenumberarea_width(self):
-        """Return line number area width"""
-        # Implemented in CodeEditor, but needed here for completion widget
-        return 0
-        
+
+
     #------Extra selections
     def get_extra_selections(self, key):
         return self.extra_selections_dict.get(key, [])
@@ -251,8 +249,13 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         
     def update_extra_selections(self):
         extra_selections = []
-        for _key, extra in self.extra_selections_dict.iteritems():
-            extra_selections.extend(extra)
+        for key, extra in list(self.extra_selections_dict.items()):
+            if key == 'current_line' or key == 'current_cell':
+                # Python 3 compatibility (weird): current line has to be 
+                # highlighted first
+                extra_selections = extra + extra_selections
+            else:
+                extra_selections += extra
         self.setExtraSelections(extra_selections)
         
     def clear_extra_selections(self, key):
@@ -266,6 +269,47 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                   self.document().isModified())
 
 
+    #------Highlight current line
+    def highlight_current_line(self):
+        """Highlight current line"""
+        selection = QTextEdit.ExtraSelection()
+        selection.format.setProperty(QTextFormat.FullWidthSelection,
+                                     to_qvariant(True))
+        selection.format.setBackground(self.currentline_color)
+        selection.cursor = self.textCursor()
+        selection.cursor.clearSelection()
+        self.set_extra_selections('current_line', [selection])
+        self.update_extra_selections()
+
+    def unhighlight_current_line(self):
+        """Unhighlight current line"""
+        self.clear_extra_selections('current_line')
+
+    #------Highlight current cell
+    def highlight_current_cell(self):
+        """Highlight current cell"""
+        selection = QTextEdit.ExtraSelection()
+        selection.format.setProperty(QTextFormat.FullWidthSelection,
+                                     to_qvariant(True))
+        selection.format.setBackground(self.currentcell_color)
+        selection.cursor, whole_file_selected, whole_screen_selected =\
+            self.select_current_cell_in_visible_portion()
+        if whole_file_selected: 
+            self.clear_extra_selections('current_cell')
+        elif whole_screen_selected:
+            if self.has_cell_separators:
+                self.set_extra_selections('current_cell', [selection])
+                self.update_extra_selections()
+            else:
+                self.clear_extra_selections('current_cell')
+        else:
+            self.set_extra_selections('current_cell', [selection])
+            self.update_extra_selections()
+
+    def unhighlight_current_cell(self):
+        """Unhighlight current cell"""
+        self.clear_extra_selections('current_cell')
+
     #------Brace matching
     def find_brace_match(self, position, brace, forward):
         start_pos, end_pos = self.BRACE_MATCHING_SCOPE
@@ -336,7 +380,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             return
         cursor.movePosition(QTextCursor.PreviousCharacter,
                             QTextCursor.KeepAnchor)
-        text = unicode(cursor.selectedText())
+        text = to_text_string(cursor.selectedText())
         pos1 = cursor.position()
         if text in (')', ']', '}'):
             pos2 = self.find_brace_match(pos1, text, forward=False)
@@ -362,11 +406,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.codecompletion_case = state
         self.completion_widget.case_sensitive = state
         
-    def set_codecompletion_single(self, state):
-        """Show single completion"""
-        self.codecompletion_single = state
-        self.completion_widget.show_single = state
-        
     def set_codecompletion_enter(self, state):
         """Enable Enter key to select completion"""
         self.codecompletion_enter = state
@@ -425,7 +464,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         min_indent = 999
         current_indent = 0
         lines = text.split(ls)
-        for i in xrange(len(lines)-1, -1, -1):
+        for i in range(len(lines)-1, -1, -1):
             line = lines[i]
             if line.strip():
                 current_indent = _indent(line)
@@ -437,10 +476,12 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
         # Remove any leading whitespace or comment lines
         # since they confuse the reserved word detector that follows below
-        first_line = lines[0].lstrip()
-        while first_line == '' or first_line[0] == '#':
-            lines.pop(0)
+        while lines:
             first_line = lines[0].lstrip()
+            if first_line == '' or first_line[0] == '#':
+                lines.pop(0)
+            else:
+                break
         
         # Add an EOL character after indentation blocks that start with some 
         # Python reserved words, so that it gets evaluated automatically
@@ -475,7 +516,9 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     def get_cell_as_executable_code(self):
         """Return cell contents as executable code"""
         start_pos, end_pos = self.__save_selection()
-        self.select_current_cell()
+        cursor, whole_file_selected = self.select_current_cell()
+        if not whole_file_selected:
+            self.setTextCursor(cursor)
         text = self.get_selection_as_executable_code()
         self.__restore_selection(start_pos, end_pos)
         return text
@@ -486,17 +529,20 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         if cursor is not None:
             cursor0 = QTextCursor(cursor)
             cursor0.select(QTextCursor.BlockUnderCursor)
-            text = unicode(cursor0.selectedText())
+            text = to_text_string(cursor0.selectedText())
         else:
-            text = unicode(block.text())
-        return text.lstrip().startswith(self.CELL_SEPARATORS)
+            text = to_text_string(block.text())
+        return text.lstrip().startswith(CELL_SEPARATORS)
 
     def select_current_cell(self):
         """Select cell under cursor
-        cell = group of lines separated by CELL_SEPARATORS"""
+        cell = group of lines separated by CELL_SEPARATORS
+        returns the textCursor and a boolean indicating if the
+        entire file is selected"""
         cursor = self.textCursor()
         cursor.movePosition(QTextCursor.StartOfBlock)
         cur_pos = prev_pos = cursor.position()
+
         # Moving to the next line that is not a separator, if we are
         # exactly at one of them
         while self.is_cell_separator(cursor):
@@ -504,7 +550,8 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             prev_pos = cur_pos
             cur_pos = cursor.position()
             if cur_pos == prev_pos:
-                return
+                return cursor, False
+        prev_pos = cur_pos
         # If not, move backwards to find the previous separator
         while not self.is_cell_separator(cursor):
             cursor.movePosition(QTextCursor.PreviousBlock)
@@ -512,7 +559,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             cur_pos = cursor.position()
             if cur_pos == prev_pos:
                 if self.is_cell_separator(cursor):
-                    return
+                    return cursor, False
                 else:
                     break
         cursor.setPosition(prev_pos)
@@ -529,11 +576,71 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                                     QTextCursor.KeepAnchor)
                 break
             prev_pos = cur_pos
-        # Do nothing if we moved from beginning to end without
-        # finding a separator
-        if cell_at_file_start and cursor.atEnd():
-            return
-        self.setTextCursor(cursor)
+        cell_at_file_end = cursor.atEnd()
+        return cursor, cell_at_file_start and cell_at_file_end
+    
+    def select_current_cell_in_visible_portion(self):
+        """Select cell under cursor in the visible portion of the file
+        cell = group of lines separated by CELL_SEPARATORS
+        returns 
+         -the textCursor
+         -a boolean indicating if the entire file is selected
+         -a boolean indicating if the entire visible portion of the file is selected"""
+        cursor = self.textCursor()
+        cursor.movePosition(QTextCursor.StartOfBlock)
+        cur_pos = prev_pos = cursor.position()
+
+        beg_pos = self.cursorForPosition(QPoint(0, 0)).position()
+        bottom_right = QPoint(self.viewport().width() - 1, 
+                              self.viewport().height() - 1)
+        end_pos = self.cursorForPosition(bottom_right).position()
+
+        # Moving to the next line that is not a separator, if we are
+        # exactly at one of them
+        while self.is_cell_separator(cursor):
+            cursor.movePosition(QTextCursor.NextBlock)
+            prev_pos = cur_pos
+            cur_pos = cursor.position()
+            if cur_pos == prev_pos:
+                return cursor, False, False
+        prev_pos = cur_pos
+        # If not, move backwards to find the previous separator
+        while not self.is_cell_separator(cursor)\
+          and cursor.position() >= beg_pos:
+            cursor.movePosition(QTextCursor.PreviousBlock)
+            prev_pos = cur_pos
+            cur_pos = cursor.position()
+            if cur_pos == prev_pos:
+                if self.is_cell_separator(cursor):
+                    return cursor, False, False
+                else:
+                    break
+        cell_at_screen_start = cursor.position() <= beg_pos
+        cursor.setPosition(prev_pos)
+        cell_at_file_start = cursor.atStart()
+        # Selecting cell header
+        if not cell_at_file_start:
+            cursor.movePosition(QTextCursor.PreviousBlock)
+            cursor.movePosition(QTextCursor.NextBlock,
+                                QTextCursor.KeepAnchor)
+        # Once we find it (or reach the beginning of the file)
+        # move to the next separator (or the end of the file)
+        # so we can grab the cell contents
+        while not self.is_cell_separator(cursor)\
+          and cursor.position() <= end_pos:
+            cursor.movePosition(QTextCursor.NextBlock,
+                                QTextCursor.KeepAnchor)
+            cur_pos = cursor.position()
+            if cur_pos == prev_pos:
+                cursor.movePosition(QTextCursor.EndOfBlock,
+                                    QTextCursor.KeepAnchor)
+                break
+            prev_pos = cur_pos
+        cell_at_file_end = cursor.atEnd()
+        cell_at_screen_end = cursor.position() >= end_pos
+        return cursor,\
+               cell_at_file_start and cell_at_file_end,\
+               cell_at_screen_start and cell_at_screen_end
 
     def go_to_next_cell(self):
         """Go to the next cell of lines"""
@@ -570,13 +677,13 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         cursor = self.textCursor()
         cursor.beginEditBlock()
         start_pos, end_pos = self.__save_selection()
-        if unicode(cursor.selectedText()):
+        if to_text_string(cursor.selectedText()):
             cursor.setPosition(end_pos)
             # Check if end_pos is at the start of a block: if so, starting
             # changes from the previous block
             cursor.movePosition(QTextCursor.StartOfBlock,
                                 QTextCursor.KeepAnchor)
-            if not unicode(cursor.selectedText()):
+            if not to_text_string(cursor.selectedText()):
                 cursor.movePosition(QTextCursor.PreviousBlock)
                 end_pos = cursor.position()
             
@@ -624,7 +731,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         cursor = self.textCursor()
         cursor.beginEditBlock()
         start_pos, end_pos = self.__save_selection()
-        if unicode(cursor.selectedText()):
+        if to_text_string(cursor.selectedText()):
             # Check if start_pos is at the start of a block
             cursor.setPosition(start_pos)
             cursor.movePosition(QTextCursor.StartOfBlock)
@@ -635,7 +742,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             # changes from the previous block
             cursor.movePosition(QTextCursor.StartOfBlock,
                                 QTextCursor.KeepAnchor)
-            if unicode(cursor.selectedText()):
+            if to_text_string(cursor.selectedText()):
                 cursor.movePosition(QTextCursor.NextBlock)
                 end_pos = cursor.position()
         else:
@@ -646,17 +753,17 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         cursor.setPosition(start_pos)
         cursor.setPosition(end_pos, QTextCursor.KeepAnchor)
         
-        sel_text = unicode(cursor.selectedText())
+        sel_text = to_text_string(cursor.selectedText())
         cursor.removeSelectedText()
         
         if after_current_line:
-            text = unicode(cursor.block().text())
+            text = to_text_string(cursor.block().text())
             start_pos += len(text)+1
             end_pos += len(text)+1
             cursor.movePosition(QTextCursor.NextBlock)
         else:
             cursor.movePosition(QTextCursor.PreviousBlock)
-            text = unicode(cursor.block().text())
+            text = to_text_string(cursor.block().text())
             start_pos -= len(text)+1
             end_pos -= len(text)+1
         cursor.insertText(sel_text)
@@ -709,38 +816,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
 
     #------Code completion / Calltips
-    def show_calltip(self, title, text, color='#2D62FF',
-                     at_line=None, at_position=None):
-        """Show calltip"""
-        if text is None or len(text) == 0:
-            return
-        # Saving cursor position:
-        if at_position is None:
-            at_position = self.get_position('cursor')
-        self.calltip_position = at_position
-        # Preparing text:
-        weight = 'bold' if self.calltip_font.bold() else 'normal'
-        size = self.calltip_font.pointSize()
-        family = self.calltip_font.family()
-        format1 = '<div style=\'font-size: %spt; color: %s\'>' % (size, color)
-        format2 = '<hr><div style=\'font-family: "%s"; font-size: %spt; font-weight: %s\'>' % (family, size, weight)
-        if isinstance(text, list):
-            text = "\n    ".join(text)
-        text = text.replace('\n', '<br>')
-        if len(text) > self.calltip_size:
-            text = text[:self.calltip_size] + " ..."
-        tiptext = format1 + ('<b>%s</b></div>' % title) \
-                  + format2 + text + "</div>"
-        # Showing tooltip at cursor position:
-        cx, cy = self.get_coordinates('cursor')
-        if at_line is not None:
-            cx = 5
-            cursor = QTextCursor(self.document().findBlockByNumber(at_line-1))
-            cy = self.cursorRect(cursor).top()
-        point = self.mapToGlobal(QPoint(cx, cy))
-        point.setX(point.x()+self.get_linenumberarea_width())
-        QToolTip.showText(point, tiptext)
-
     def hide_tooltip_if_necessary(self, key):
         """Hide calltip when necessary"""
         try:
@@ -748,7 +823,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             before = self.is_cursor_before(self.calltip_position,
                                            char_offset=1)
             other = key in (Qt.Key_ParenRight, Qt.Key_Period, Qt.Key_Tab)
-            if calltip_char not in ('?','(') or before or other:
+            if calltip_char not in ('?', '(') or before or other:
                 QToolTip.hideText()
         except (IndexError, TypeError):
             QToolTip.hideText()
@@ -769,9 +844,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.completion_text = completion_text
         # Sorting completion list (entries starting with underscore are 
         # put at the end of the list):
-        underscore = set([comp for comp in completions if comp.startswith('_')])
-        completions = sorted(set(completions)-underscore, key=string.lower)+\
-                      sorted(underscore, key=string.lower)
+        underscore = set([comp for comp in completions
+                          if comp.startswith('_')])
+        completions = sorted(set(completions)-underscore, key=str_lower)+\
+                      sorted(underscore, key=str_lower)
         self.show_completion_widget(completions, automatic=automatic)
         
     def select_completion_list(self):
@@ -857,6 +933,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     def mousePressEvent(self, event):
         """Reimplement Qt method"""
         if os.name != 'posix' and event.button() == Qt.MidButton:
+            self.calltip_widget.hide()
             self.setFocus()
             event = QMouseEvent(QEvent.MouseButtonPress, event.pos(),
                                 Qt.LeftButton, Qt.LeftButton, Qt.NoModifier)
@@ -864,6 +941,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             QPlainTextEdit.mouseReleaseEvent(self, event)
             self.paste()
         else:
+            self.calltip_widget.hide()
             QPlainTextEdit.mousePressEvent(self, event)
 
     def focusInEvent(self, event):
@@ -889,6 +967,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                     self.emit(SIGNAL("zoom_in()"))
                 return
         QPlainTextEdit.wheelEvent(self, event)
+        self.highlight_current_cell()
 
 
 class QtANSIEscapeCodeHandler(ANSIEscapeCodeHandler):
@@ -1143,4 +1222,4 @@ class ConsoleBaseWidget(TextEditBaseWidget):
                               light_background=self.light_background,
                               is_default=style is self.default_style)
         self.ansi_handler.set_base_format(self.default_style.format)
-        
\ No newline at end of file
+        
