@@ -23,6 +23,7 @@ from spyderlib.baseconfig import _
 from spyderlib.utils.misc import get_common_path
 from spyderlib.utils.qthelpers import (add_actions, create_toolbutton,
                                        create_action, get_icon)
+from spyderlib.py3compat import PY2, to_text_string
 
 
 class TabBar(QTabBar):
@@ -49,9 +50,14 @@ class TabBar(QTabBar):
             drag = QDrag(self)
             mimeData = QMimeData()
             # Converting id's to long to avoid an OverflowError with PySide
-            ancestor_id = long(id(self.ancestor))
-            parent_widget_id = long(id(self.parentWidget()))
-            self_id = long(id(self))
+            if PY2:
+                ancestor_id = long(id(self.ancestor))
+                parent_widget_id = long(id(self.parentWidget()))
+                self_id = long(id(self))
+            else:
+                ancestor_id = id(self.ancestor)
+                parent_widget_id = id(self.parentWidget())
+                self_id = id(self)
             mimeData.setData("parent-id", QByteArray.number(ancestor_id))
             mimeData.setData("tabwidget-id",
                              QByteArray.number(parent_widget_id))
@@ -137,9 +143,9 @@ class BaseTabs(QTabWidget):
         dirnames = []
         for index in range(self.count()):
             if self.menu_use_tooltips:
-                text = unicode(self.tabToolTip(index))
+                text = to_text_string(self.tabToolTip(index))
             else:
-                text = unicode(self.tabText(index))
+                text = to_text_string(self.tabText(index))
             names.append(text)
             if osp.isfile(text):
                 # Testing if tab names are filenames
@@ -177,7 +183,7 @@ class BaseTabs(QTabWidget):
         assert all(key in (Qt.TopLeftCorner, Qt.TopRightCorner)
                    for key in corner_widgets)
         self.corner_widgets.update(corner_widgets)
-        for corner, widgets in self.corner_widgets.iteritems():
+        for corner, widgets in list(self.corner_widgets.items()):
             cwidget = QWidget()
             cwidget.hide()
             prev_widget = self.cornerWidget(corner)
