@@ -38,6 +38,7 @@ from spyderlib.baseconfig import get_conf_path, get_translation
 from spyderlib.widgets.texteditor import TextEditor
 from spyderlib.widgets.comboboxes import PythonModulesComboBox
 from spyderlib.widgets.externalshell import baseshell
+from spyderlib.py3compat import to_text_string, getcwd
 _ = get_translation("p_profiler", dirname="spyderplugins")
 
 
@@ -50,7 +51,7 @@ class ProfilerWidget(QWidget):
     """
     Profiler widget
     """
-    DATAPATH = get_conf_path('.profiler.results')
+    DATAPATH = get_conf_path('profiler.results')
     VERSION = '0.0.1'
     
     def __init__(self, parent, max_entries=100):
@@ -166,7 +167,7 @@ class ProfilerWidget(QWidget):
     def select_file(self):
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
         filename, _selfilter = getopenfilename(self, _("Select Python script"),
-                           os.getcwdu(), _("Python scripts")+" (*.py ; *.pyw)")
+                           getcwd(), _("Python scripts")+" (*.py ; *.pyw)")
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
         if filename:
             self.analyze(filename)
@@ -182,7 +183,7 @@ class ProfilerWidget(QWidget):
                        readonly=True, size=(700, 500)).exec_()
 
     def start(self, wdir=None, args=None, pythonpath=None):
-        filename = unicode(self.filecombo.currentText())
+        filename = to_text_string(self.filecombo.currentText())
         if wdir is None:
             wdir = self._last_wdir
             if wdir is None:
@@ -212,7 +213,8 @@ class ProfilerWidget(QWidget):
         self.connect(self.stop_button, SIGNAL("clicked()"), self.process.kill)
 
         if pythonpath is not None:
-            env = [unicode(_pth) for _pth in self.process.systemEnvironment()]
+            env = [to_text_string(_pth)
+                   for _pth in self.process.systemEnvironment()]
             baseshell.add_pathlist_to_PYTHONPATH(env, pythonpath)
             self.process.setEnvironment(env)
         
@@ -250,13 +252,13 @@ class ProfilerWidget(QWidget):
             self.process.setReadChannel(QProcess.StandardError)
         else:
             self.process.setReadChannel(QProcess.StandardOutput)
-        bytes = QByteArray()
+        qba = QByteArray()
         while self.process.bytesAvailable():
             if error:
-                bytes += self.process.readAllStandardError()
+                qba += self.process.readAllStandardError()
             else:
-                bytes += self.process.readAllStandardOutput()
-        text = unicode( locale_codec.toUnicode(bytes.data()) )
+                qba += self.process.readAllStandardOutput()
+        text = to_text_string( locale_codec.toUnicode(qba.data()) )
         if error:
             self.error_output += text
         else:
@@ -282,7 +284,7 @@ class ProfilerWidget(QWidget):
         self.log_button.setEnabled(self.output is not None \
                                    and len(self.output) > 0)
         self.kill_if_running()
-        filename = unicode(self.filecombo.currentText())
+        filename = to_text_string(self.filecombo.currentText())
         if not filename:
             return
 
@@ -365,17 +367,11 @@ class ProfilerDataTree(QTreeWidget):
 
     def find_root(self):
         """Find a function without a caller"""
-        for func, (cc, nc, tt, ct, callers) in self.stats.iteritems():
-            if not callers and self.find_callees(func):
+        self.profdata.sort_stats("cumulative")
+        for func in self.profdata.fcn_list:
+            if ('~', 0, '<built-in method exec>') != func:
                 return func
     
-    def find_root_skip_profilerfuns(self):
-        """Define root ignoring profiler-specific functions."""
-        # FIXME: this is specific to module 'profile' and has to be
-        #        changed for cProfile
-        #return ('', 0, 'execfile')     # For 'profile' module   
-        return ('~', 0, '<execfile>')     # For 'cProfile' module   
-    
     def find_callees(self, parent):
         """Find all functions called by (parent) function."""
         # FIXME: This implementation is very inneficient, because it
@@ -388,7 +384,6 @@ class ProfilerDataTree(QTreeWidget):
         self.setItemsExpandable(True)
         self.setSortingEnabled(False)
         rootkey = self.find_root()  # This root contains profiler overhead
-#        rootkey = self.find_root_skip_profilerfuns()
         if rootkey:
             self.populate_tree(self, self.find_callees(rootkey))
             self.resizeColumnToContents(0)
@@ -438,11 +433,11 @@ class ProfilerDataTree(QTreeWidget):
             child_item.setData(1, Qt.DisplayRole, '%.3f' % cum_time)
             child_item.setTextAlignment(1, Qt.AlignCenter)
 
-            child_item.setToolTip(2,_('Local time in function '\
+            child_item.setToolTip(2, _('Local time in function '\
                                       '(not in sub-functions)'))
             #child_item.setData(2, Qt.DisplayRole, loc_time)
             child_item.setData(2, Qt.DisplayRole, '%.3f' % loc_time)
-            child_item.setTextAlignment(2,Qt.AlignCenter)
+            child_item.setTextAlignment(2, Qt.AlignCenter)
 
             child_item.setToolTip(3, _('Total number of calls '\
                                        '(including recursion)'))
@@ -462,7 +457,7 @@ class ProfilerDataTree(QTreeWidget):
                     self.populate_tree(child_item, callees)
                 elif callees:
                     child_item.setChildIndicatorPolicy(child_item.ShowIndicator)
-                    self.items_to_be_shown[child_item] = callees
+                    self.items_to_be_shown[id(child_item)] = callees
             self.item_depth -= 1
         
     def item_activated(self, item):
@@ -471,8 +466,8 @@ class ProfilerDataTree(QTreeWidget):
                            filename, line_number, '')
             
     def item_expanded(self, item):
-        if item.childCount() == 0 and item in self.items_to_be_shown:
-            callees = self.items_to_be_shown[item]
+        if item.childCount() == 0 and id(item) in self.items_to_be_shown:
+            callees = self.items_to_be_shown[id(item)]
             self.populate_tree(item, callees)
     
     def is_recursive(self, child_item):
@@ -528,7 +523,7 @@ def test():
     widget.show()
     #widget.analyze(__file__)
     widget.analyze(osp.join(osp.dirname(__file__), os.pardir, os.pardir,
-                            'rope_profiling', 'test.py'))
+                            'spyderlib/widgets', 'texteditor.py'))
     sys.exit(app.exec_())
     
 if __name__ == '__main__':
