@@ -23,7 +23,6 @@ import sys
 import os
 import os.path as osp
 import time
-import cPickle
 import re
 import subprocess
 
@@ -37,6 +36,7 @@ from spyderlib.widgets.onecolumntree import OneColumnTree
 from spyderlib.widgets.texteditor import TextEditor
 from spyderlib.widgets.comboboxes import (PythonModulesComboBox,
                                           is_module_or_package)
+from spyderlib.py3compat import to_text_string, getcwd, pickle
 _ = get_translation("p_pylint", dirname="spyderplugins")
 
 
@@ -76,7 +76,7 @@ class ResultsTree(OneColumnTree):
         
     def activated(self, item):
         """Double-click event"""
-        data = self.data.get(item)
+        data = self.data.get(id(item))
         if data is not None:
             fname, lineno = data
             self.parent().emit(SIGNAL("edit_goto(QString,int,QString)"),
@@ -150,14 +150,14 @@ class ResultsTree(OneColumnTree):
                     text = "%d : %s" % (lineno, message)
                 msg_item = QTreeWidgetItem(parent, [text], QTreeWidgetItem.Type)
                 msg_item.setIcon(0, get_icon('arrow.png'))
-                self.data[msg_item] = (modname, lineno)
+                self.data[id(msg_item)] = (modname, lineno)
 
 
 class PylintWidget(QWidget):
     """
     Pylint widget
     """
-    DATAPATH = get_conf_path('.pylint.results')
+    DATAPATH = get_conf_path('pylint.results')
     VERSION = '1.1.0'
     
     def __init__(self, parent, max_entries=100):
@@ -172,10 +172,10 @@ class PylintWidget(QWidget):
         self.rdata = []
         if osp.isfile(self.DATAPATH):
             try:
-                data = cPickle.loads(file(self.DATAPATH, 'U').read())
+                data = pickle.loads(open(self.DATAPATH, 'rb').read())
                 if data[0] == self.VERSION:
                     self.rdata = data[1:]
-            except EOFError:
+            except (EOFError, ImportError):
                 pass
 
         self.filecombo = PythonModulesComboBox(self)
@@ -198,7 +198,7 @@ class PylintWidget(QWidget):
         self.connect(self.filecombo, SIGNAL('valid(bool)'), self.show_data)
 
         browse_button = create_toolbutton(self, icon=get_icon('fileopen.png'),
-                               tip=_('Select Python script'),
+                               tip=_('Select Python file'),
                                triggered=self.select_file)
 
         self.ratelabel = QLabel()
@@ -206,7 +206,7 @@ class PylintWidget(QWidget):
         self.log_button = create_toolbutton(self, icon=get_icon('log.png'),
                                     text=_("Output"),
                                     text_beside_icon=True,
-                                    tip=_("Complete Pylint output"),
+                                    tip=_("Complete output"),
                                     triggered=self.show_log)
         self.treewidget = ResultsTree(self)
         
@@ -241,7 +241,7 @@ class PylintWidget(QWidget):
                 # Pylint is installed but pylint script is not in PATH
                 # (AFAIK, could happen only on Windows)
                 text = _('Pylint script was not found. Please add "%s" to PATH.')
-                text = unicode(text) % osp.join(sys.prefix, "Scripts")
+                text = to_text_string(text) % osp.join(sys.prefix, "Scripts")
             else:
                 text = _('Please install <b>pylint</b>:')
                 url = 'http://www.logilab.fr'
@@ -253,7 +253,7 @@ class PylintWidget(QWidget):
     def analyze(self, filename):
         if PYLINT_PATH is None:
             return
-        filename = unicode(filename) # filename is a QString instance
+        filename = to_text_string(filename) # filename is a QString instance
         self.kill_if_running()
         index, _data = self.get_data(filename)
         if index is None:
@@ -267,8 +267,8 @@ class PylintWidget(QWidget):
             
     def select_file(self):
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
-        filename, _selfilter = getopenfilename(self, _("Select Python script"),
-                           os.getcwdu(), _("Python scripts")+" (*.py ; *.pyw)")
+        filename, _selfilter = getopenfilename(self, _("Select Python file"),
+                           getcwd(), _("Python files")+" (*.py ; *.pyw)")
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
         if filename:
             self.analyze(filename)
@@ -300,7 +300,7 @@ class PylintWidget(QWidget):
     def save(self):
         while len(self.rdata) > self.max_entries:
             self.rdata.pop(-1)
-        cPickle.dump([self.VERSION]+self.rdata, file(self.DATAPATH, 'w'))
+        pickle.dump([self.VERSION]+self.rdata, open(self.DATAPATH, 'wb'), 2)
         
     def show_log(self):
         if self.output:
@@ -308,7 +308,7 @@ class PylintWidget(QWidget):
                        readonly=True, size=(700, 500)).exec_()
         
     def start(self):
-        filename = unicode(self.filecombo.currentText())
+        filename = to_text_string(self.filecombo.currentText())
         
         self.process = QProcess(self)
         self.process.setProcessChannelMode(QProcess.SeparateChannels)
@@ -354,13 +354,13 @@ class PylintWidget(QWidget):
             self.process.setReadChannel(QProcess.StandardError)
         else:
             self.process.setReadChannel(QProcess.StandardOutput)
-        bytes = QByteArray()
+        qba = QByteArray()
         while self.process.bytesAvailable():
             if error:
-                bytes += self.process.readAllStandardError()
+                qba += self.process.readAllStandardError()
             else:
-                bytes += self.process.readAllStandardOutput()
-        text = unicode( locale_codec.toUnicode(bytes.data()) )
+                qba += self.process.readAllStandardOutput()
+        text = to_text_string( locale_codec.toUnicode(qba.data()) )
         if error:
             self.error_output += text
         else:
@@ -421,7 +421,7 @@ class PylintWidget(QWidget):
                 previous = self.output[i_prun+len(txt_prun):i_prun_end]
             
         
-        filename = unicode(self.filecombo.currentText())
+        filename = to_text_string(self.filecombo.currentText())
         self.set_data(filename, (time.localtime(), rate, previous, results))
         self.output = self.error_output + self.output
         self.show_data(justanalyzed=True)
@@ -438,7 +438,7 @@ class PylintWidget(QWidget):
         self.log_button.setEnabled(self.output is not None \
                                    and len(self.output) > 0)
         self.kill_if_running()
-        filename = unicode(self.filecombo.currentText())
+        filename = to_text_string(self.filecombo.currentText())
         if not filename:
             return
         
